local fbversion = 1.30
--[[

Flex Bar 
        Author:         Sean Kennedy (derived from Telo's work)
        Version:        1.30:  01/30/2004
				Code is now in 4 modules:
				FlexBar.lua is primarily OnFoo functions
				FlexBar_SlashCommands.lua implements the slash commands
				FlexBar_HelperFunctions.lua are any functions needed for FB
					that are not generalized functions
				WoW_UtilityClasses.lua are generally useful functions.
				
        ToDo:           Add gui (when time permits
        Last Modified  :  
		12/26/2004	Complete rewrite to facilitate new features and
					maintainablity
		01/11/2005	Fixed 3 errors:
					1, hotkeys were not getting the right button - fixed
					2, the id map was being set incorrectly for druids and warriors
					3, after group auto-arrange group bounds were off.
					4, fixed ingame help for group auto-arrange
					
					Also added code to stop using implicit buttons and use an explicit button
					in  case this was getting munged somehow.  Still desperately seeking 
					cause of Mac crashes - hampered by lack of a mac :/
		01/12/2005	1. Increased LoadProfile timer.  Was 10 seconds during development,
					dropped to 5 just two days to release -- seems to cause some shifting
					of buttons on load.
					2. Fixed small bug with key-binding text
					3. Fixed bug with setting up event for Use On=
					4. Fixed bug when auto-arranging buttons
					5. Fixed bug where FBEvents.n wasn't initialized - found out due to Cosmos.
					This is not a cosmos bug, but cosmos did something to point out a bug I'd 
					been getting away with.
					6. Made events and targets case insensitive.
					7. Added FlexBar_Config.lua file and LoadConfig command
					8. LoseAura was incorrectly being raised as LostAura.  For consistency, fixed that
					9. 7:00 pm - target is actually case insensitive now
					10. bug with buttons staying fixed to anchor after ungroup fixed.
		01/13/2005	1.  FIXED MAC CRASH
					A huge thank you to Dagar for all his help in tracking this down.
		01/16/2005	1.  Fixed hotkey problem that especially effected warriors
					2.  Added far more bounds checking to parameters with descriptive error messages
					3.  Added IsUsable, NotUsable, ManaAbove##, ManaBelow##, HealthFull, ManaFull
					     Mana events work for rage.  Probably energy too.
		01/25/2005	1.  Load profile timer now is set in Player Entering World event to ensure that settings
					    aren't lost
					2.  Added the reset parameter to shade to allow WoW's default shading on low mana
					     to work again.
					3.  You can now specify '' or even no text at all in /flexbar text to get rid of the text there.
					4.  You can now turn off button tooltips with /flexbar tooltip state='off' and state='on to 
					     turn them back on.
					5.  Some of the group bounds issues after auto-arranging groups should be solved.
					6.  Added leftbuttondown, up and click events when using a hotkey to activate a button
					7.  Fixed positioning on move to mouse and moverel when UI scale is not 1
		01/30/2005	1.  Made 2 changes to try and fix the lost configuration bug:
						A.  Added a 2nd check in Loadprofile for variables being loaded.  If they are not, it restarts the timer
						B.  Added the SafeLoad command, which requires manual loading of profiles
		01/30/2005	1.  Added SaveProfile and LoadProfile to allow backing up and transporting full profiles across characters
					    on the same account.
		02/02/2005	1.  Fixed remap toggle ability.
					2.  Added checks to reduce FPS slowdonw when lots of buttons are visible.
						Default blizzard code reshades visible buttons every update, regardless of 
						whether they actually change state.  With lots of buttons on screen this may
						be computationally expensive.  Added a check to see if the button state had
						actually changed before shading.
					3.  Fixed the bug where I was incorrectly applying scale in the case of UIScale < 1.
						This fix will result in the buttons being smaller at the same scaling if you have
						Use UI Scale checked.
					4.  Added an optional reset='true' to scale
					5.  Fixed Horizontal/Vertical group to take UI scale into account.  If you change
					     UIScale, you will need to re-issue the auto-arrange commands to make them
					     look right.
		02/04/2005	1.  Changed the Profile Auto-load code to take into account new information regarding
					    checking for a valid character name and the UnregisterEvent() bug.  
						A.  The lost config bug should be gone.
						B.  The problems people experienced with the profile never loading
						     due to a conflict with other mods should be gone.
						C.   In  the event that VARIABLES_LOADED doesn't fire due to the above
						      Blizzard bug, a dialog box will show allowing the user to manually load
						      their profile.
						
]]

--------------------------------------------------------------------------------------------------
-- Config variables - Saved
	FBSavedProfile = {}
	FBFirstUse = 0;
	FBVerbose = nil;
	FBNoTooltips = nil;
	FBSafeLoad = nil;
-------------------------------------------------------------------------------------------------
-- Local variables
-- This is the last known cursor location (so we can see if it entered/left a group bounding box
	FBCursorLoc = {};
	FBCursorLoc.x=0;
	FBCursorLoc.y=0;
-- id - Button map - must be global to be accessed in XML file
--	FBIDToButton = {}
-- Group data - global.  Used to speed up bounds checking
	FBGroupData = {}
-- Constant for number of buttons - must be global
	FBNumButtons = 96;
-- Utility object
	local util = Utility_Class:New()
-- Command object and /command setup - must be global
	FBcmd = Command_Class:New("FlexBar")
	SLASH_FlexBarCOMMAND1 = "/flexbar"
-- Timers
	FBTimers = {}
-- Getting the mouse button pressed in OnClick is flaky - store the last mousebutton down here
	FBLastButtonDown = "";
-- Last hp percents for various units
	FBLastHealth = {}
	FBLastMana = {}
	FBLastRage = {}
	FBLastEnergy = {}
-- Current events list copied from FBSavedProfile
	FBEvents = {}
	FBEvents.n = 0
--
	FBVariablesLoaded = nil
	FBPlayerNameValid = nil
--------------------------------------------------------------------------------------------------
-- OnFoo functions

function FlexBar_OnLoad()
	-- Set up slash commands
	FB_Command_AddCommands(FBcmd)
	-- Register for events
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_ENTER_COMBAT");
	this:RegisterEvent("PLAYER_LEAVE_COMBAT");
	this:RegisterEvent("PLAYER_REGEN_ENABLED");
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF");
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS");
	this:RegisterEvent("CHAT_MSG_COMBAT_SELF_MISSES");
	this:RegisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES");
	this:RegisterEvent("UNIT_HEALTH")
	this:RegisterEvent("UNIT_MANA")
	this:RegisterEvent("UNIT_RAGE")
	this:RegisterEvent("UNIT_ENERGY")
	this:RegisterEvent("ACTIONBAR_PAGE_CHANGED")
	this:RegisterEvent("PLAYER_ENTERING_WORLD")
	this:RegisterEvent("UNIT_NAME_UPDATE")
end

function FlexBar_OnEvent()
	-- Saved Variables available here
	if (event == "VARIABLES_LOADED") then
		if FBFirstUse == 0 then
			FlexBar_LoadDefaults();
		end
		util:Print(format("FlexBar V%.2f loaded.  Type /flexbar for useage", fbversion));
		FBLoadProfileDialog:Hide()
		if FBSafeLoad then
			LoadProfileButton:Show()
		else
			FBTimers["loadprofile"] = Timer_Class:New(10, nil, 
				"Flexbar settings loading", nil, FB_LoadProfile)
		end
	elseif (event == "PLAYER_ENTER_COMBAT") then
	-- self explanatory - raise event for player initiating auto-attack
		FB_RaiseEvent("EnterCombat")
	elseif (event == "PLAYER_LEAVE_COMBAT") then
	-- raise event for player turning autoattack off
		FB_RaiseEvent("LeaveCombat")
	elseif (event == "PLAYER_REGEN_ENABLED") then
	-- raise event for lose aggro (which is when regen is re-enabled)
		FB_RaiseEvent("LoseAggro")
	elseif (event == "PLAYER_REGEN_DISABLED") then
	-- raise event for gain aggro (player regen is disabled when you have aggro)
		FB_RaiseEvent("GainAggro")
	elseif (event == "CHAT_MSG_SPELL_AURA_GONE_SELF") then
	-- lost buff - raise event.  Note, losing buffs to dieing doesn't trigger this.
	-- also, some forms do not trigger this when they fade apparently
		FB_RaiseEvent("LoseBuff", string.gsub(arg1, " fades from you.", ""))
	elseif (event == "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS") then
	-- Not all forms trigger this: Caster form being one
		arg1 = string.gsub(arg1, "You gain ", "")
		FB_RaiseEvent("GainBuff", string.sub(arg1,1,-2))
	elseif (event == "CHAT_MSG_COMBAT_SELF_MISSES") then
	-- pretty sure all these are right - if anyone has trouble getting the right reactions I'll revisit
		if strfind(arg1, "dodges") then
			FB_RaiseEvent("PlayerMiss", "dodge")
		elseif strfind(arg1, "parries") then
			FB_RaiseEvent("PlayerMiss", "parry")
		elseif strfind(arg1, "blocks") then
			FB_RaiseEvent("PlayerMiss", "block")
		else
			FB_RaiseEvent("PlayerMiss", "miss")
		end
	elseif (event == "CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES") then
	-- Same as above
		if strfind(arg1, "dodge") then
			FB_RaiseEvent("TargetMiss", "dodge")
		elseif strfind(arg1, "parry") then
			FB_RaiseEvent("TargetMiss", "parry")
		elseif strfind(arg1, "block") then
			FB_RaiseEvent("TargetMiss", "block")
		else
			FB_RaiseEvent("TargetMiss", "miss")
		end
	elseif (event == "UNIT_HEALTH") then
	-- raise events for the health of available unit codes dropping by 10% increments, and
	-- climbing by the same increments.  Available units are player, party1-party4, pet, target, mouseover
		local hppercent = UnitHealth(arg1)/UnitHealthMax(arg1) * 100
		if FBLastHealth[arg1] then
			local index
			for index = 100, 10, -10 do
				if FBLastHealth[arg1] >= index and hppercent < index then
					FB_RaiseEvent("HealthBelow" .. index, arg1)
				end
			end
			for index = 10, 90, 10 do
				if FBLastHealth[arg1] <= index and hppercent > index then
					FB_RaiseEvent("HealthAbove" .. index, arg1)
				end
			end
			if FBLastHealth[arg1] <=99 and hppercent > 99 then
				FB_RaiseEvent("HealthFull", arg1)
			end
		end
		FBLastHealth[arg1] = hppercent
	elseif (event == "UNIT_MANA") or (event == "UNIT_RAGE") or (event == "UNIT_ENERGY") then
	-- raise events for the health of available unit codes dropping by 10% increments, and
	-- climbing by the same increments.  Available units are player, party1-party4, pet, target, mouseover
		local manapercent = UnitMana(arg1)/UnitManaMax(arg1) * 100
		if FBLastMana[arg1] then
			local index
			for index = 100, 10, -10 do
				if FBLastMana[arg1] >= index and manapercent < index then
					FB_RaiseEvent("ManaBelow" .. index, arg1)
				end
			end
			for index = 10, 90, 10 do
				if FBLastMana[arg1] <= index and manapercent > index then
					FB_RaiseEvent("ManaAbove" .. index, arg1)
				end
			end
			if FBLastMana[arg1] <=99 and manapercent > 99 then
				FB_RaiseEvent("ManaFull", arg1)
			end
		end
		FBLastMana[arg1] = manapercent	elseif (event == "ACTIONBAR_PAGE_CHANGED") then
	-- Action bar changed - allows syncing group to action bar
		FB_RaiseEvent("ActionBarPage", CURRENT_ACTIONBAR_PAGE)
	elseif (event == "some event") then
	-- my tester for new events to see what I have to work with.
		util:Print(event)
		util:Print(arg1)
		util:Print(arg2)
		util:Print(arg3)
		util:Print(arg4)
		util:Print(arg5)
	end
end

function Flexbar_OnUpdate()

-- Update any timers we have running.
	local name, timer
	for name, timer in pairs(FBTimers) do
		timer:Update(arg1)
	end
end

function FlexBarButtonDown(buttonnum)
-- Modified Blizzard code (removed bonus action check)
-- appears to simply change the look of the button to reflect that 
-- it is pushed.

-- originally coded to use the button ID - but why when this is only called from bindings?
	local button = getglobal("FlexBarButton".. buttonnum);
	if ( button:GetButtonState() == "NORMAL" ) then
		button:SetButtonState("PUSHED");
	end
end

function FlexBarButtonUp(buttonnum)
-- Modified Blizzard code (removed bonus action check)
-- besides the appearance change, not sure what they are doing here.
-- same as above
	local button = getglobal("FlexBarButton".. buttonnum);
	if ( button:GetButtonState() == "PUSHED" ) then
		button:SetButtonState("NORMAL");
		-- Used to save a macro
		MacroFrame_EditMacro();
		UseAction(FlexBarButton_GetID(button), 0);
		if ( IsCurrentAction(FlexBarButton_GetID(button)) ) then
			button:SetChecked(1);
		else
			button:SetChecked(0);
		end
	end
end

function FlexBarButton_OnLoad()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- FlexBar code - saving button state on the button
	button.FB = {}
	button.FB.State = {}
	-- start hidden, otherwise all buttons visible until loadprofile called
	if button:GetParent() ~= "UIParent" then button:GetParent():Hide() end
-- Blizzard code:  Called as each button is loaded.
	button.showgrid = 1;
	button.flashing = 0;
	button.flashtime = 0;
	FlexBarButton_Update();
-- Went nuts trying to figure out why I couldn't get clicks and drags
-- in my handle frames - I didn't do this 
	button:RegisterForDrag("LeftButton", "RightButton");
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	button:RegisterEvent("VARIABLES_LOADED");
	button:RegisterEvent("ACTIONBAR_SHOWGRID");
	button:RegisterEvent("ACTIONBAR_HIDEGRID");
	button:RegisterEvent("ACTIONBAR_PAGE_CHANGED");
	button:RegisterEvent("ACTIONBAR_SLOT_CHANGED");
	button:RegisterEvent("ACTIONBAR_UPDATE_STATE");
	button:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
	button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
	button:RegisterEvent("UPDATE_INVENTORY_ALERTS");
	button:RegisterEvent("PLAYER_AURAS_CHANGED");
	button:RegisterEvent("PLAYER_TARGET_CHANGED");
	button:RegisterEvent("UNIT_AURASTATE");
	button:RegisterEvent("UNIT_INVENTORY_CHANGED");
	button:RegisterEvent("CRAFT_SHOW");
	button:RegisterEvent("CRAFT_CLOSE");
	button:RegisterEvent("TRADE_SKILL_SHOW");
	button:RegisterEvent("TRADE_SKILL_CLOSE");
	button:RegisterEvent("UNIT_HEALTH");
	button:RegisterEvent("UNIT_MANA");
	button:RegisterEvent("UNIT_RAGE");
	button:RegisterEvent("UNIT_FOCUS");
	button:RegisterEvent("UNIT_ENERGY");
	button:RegisterEvent("PLAYER_ENTER_COMBAT");
	button:RegisterEvent("PLAYER_LEAVE_COMBAT");
	button:RegisterEvent("PLAYER_COMBO_POINTS");
	button:RegisterEvent("UPDATE_BINDINGS");
	button:RegisterEvent("START_AUTOREPEAT_SPELL");
	button:RegisterEvent("STOP_AUTOREPEAT_SPELL");
	button:RegisterEvent("PLAYER_ENTERING_WORLD");
	FlexBarButton_UpdateHotkeys();
this = button
end

function FlexBarButton_UpdateHotkeys()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- This sets the hotkey text.  Originally borrowed from Telo's SideBar,
-- optionally puts binding or id in depending on user selection.
	local name = button:GetName();
	local hotkey = getglobal(name.."HotKey");
	local s, e, id = string.find(name, "^FlexBarButton(%d+)$");
	local action = "FLEXACTIONBUTTON"..id;
	local text = KeyBindingFrame_GetLocalizedName(GetBindingKey(action), "KEY_");
	
	text = string.gsub(text, "CTRL--", "C-");
	text = string.gsub(text, "ALT--", "A-");
	text = string.gsub(text, "SHIFT--", "S-");
	text = string.gsub(text, "Num Pad", "NP");
	text = string.gsub(text, "Backspace", "Bksp");
	text = string.gsub(text, "Spacebar", "Space");
	text = string.gsub(text, "Page", "Pg");
	text = string.gsub(text, "Down", "Dn");
	text = string.gsub(text, "Arrow", "");
	text = string.gsub(text, "Insert", "Ins");
	text = string.gsub(text, "Delete", "Del");
	
	if button.FB.State["hotkeytext"] == nil then button.FB.State["hotkeytext"] = "" end
	-- going to add %s for slots free for bags here later
	if button.FB.State["hotkeytext"] == "%b" then
		hotkey:SetText(text);
	elseif button.FB.State["hotkeytext"] == "%c" then
		hotkey:SetText("**")
	else
		hotkey:SetText(format(button.FB.State["hotkeytext"],FB_GetButtonNum(button)));
	end
this = button
end

function FlexBarButton_Update()
-- When called manually, this can change - changed code to use button instead of this
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code
	-- Determine whether or not the button should be flashing or not since the button may have missed the enter combat event
	local buttonID = FlexBarButton_GetID(button);
	if ( IsAttackAction(buttonID) and IsCurrentAction(buttonID) ) then
		IN_ATTACK_MODE = 1;
	else
		IN_ATTACK_MODE = nil;
	end
	IN_AUTOREPEAT_MODE = IsAutoRepeatAction(buttonID);
	
	local icon = getglobal(button:GetName().."Icon");
	local buttonCooldown = getglobal(button:GetName().."Cooldown");
	local texture = GetActionTexture(FlexBarButton_GetID(button));
	if ( texture ) then
		icon:SetTexture(texture);
		icon:Show();
		button.rangeTimer = TOOLTIP_UPDATE_TIME;
		button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot2");
	else
		icon:Hide();
		buttonCooldown:Hide();
		button.rangeTimer = nil;
		button:SetNormalTexture("Interface\\Buttons\\UI-Quickslot");
		getglobal(button:GetName().."HotKey"):SetVertexColor(0.6, 0.6, 0.6);
	end
	FlexBarButton_UpdateCount();
	if ( HasAction(FlexBarButton_GetID(button)) ) then
		button:Show();
		-- somewhere right before here, this gets reset to whatever 
		-- has focus -- so put it back.  I don't know if this causes problems
		-- but it is certainly the first place to look.  This only happens with
		-- remapping an empty button with showgrid=0 to a non-empty button.
		this=button
		FlexBarButton_UpdateUsable();
		this=button
		FlexBarButton_UpdateCooldown();
	elseif ( button.showgrid == 0 ) then
		button:Hide();
	else
		getglobal(button:GetName().."Cooldown"):Hide();
	end

	if ( IN_ATTACK_MODE or IN_AUTOREPEAT_MODE ) then
		FlexBarButton_StartFlash();
	else
		FlexBarButton_StopFlash();
	end

	if ( GameTooltip:IsOwned(button) ) then
		FlexBarButton_SetTooltip();
	else
		button.updateTooltip = nil;
	end

	-- Update Macro Text
	local macroName = getglobal(button:GetName().."Name");
	macroName:SetText(GetActionText(FlexBarButton_GetID(button)))
this = button
end

function FlexBarButton_ShowGrid()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code;  Show button even if it doesn't have an action associated with it.
	button.showgrid = button.showgrid+1;
	getglobal(button:GetName().."NormalTexture"):SetVertexColor(1.0, 1.0, 1.0);
	button:Show();
this = button
end

function FlexBarButton_HideGrid()	
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code: Hide button if it doesn't have an action associated with it
	button.showgrid = button.showgrid-1;
	if ( button.showgrid == 0 and not HasAction(FlexBarButton_GetID(button)) ) then
		button:Hide();
	end
this = button
end

function FlexBarButton_UpdateState()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code: Purpose unknown at button time
	if ( IsCurrentAction(FlexBarButton_GetID(button)) or IsAutoRepeatAction(FlexBarButton_GetID(button)) ) then
		button:SetChecked(1);
	else
		button:SetChecked(0);
	end
this = button
end

function FlexBarButton_UpdateUsable()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code:
	local icon = getglobal(button:GetName().."Icon");
	local normalTexture = getglobal(button:GetName().."NormalTexture");
-- Mana check available here - also has potential
	local isUsable, notEnoughMana = IsUsableAction(FlexBarButton_GetID(button));
-- Hotkey is the text displayed in the upper right corner.
	local count = getglobal(button:GetName().."HotKey");
	local text = count:GetText();
-- Here's the test for action in range - for further enhancements that let the player know button better
	local inRange = IsActionInRange(FlexBarButton_GetID(button));
	if( inRange == 0 ) then
		count:SetVertexColor(1.0, 0.1, 0.1);
	else
		count:SetVertexColor(0.6, 0.6, 0.6);
	end
	-- Default blizz code resets the vertex color every pass through, regardless of whether the state
	-- changed.  This may be a cause of FPS reduction when people have lots of buttons visible.  Added checks
	-- so coloring will only occur on an actual change of state.
	if ( isUsable ) then
-- Telo added this code - If the button has hotkey text it colors like normal on in/out of range
-- w/o hotkey text it colors the entire button.  
		if ( (not text or text == "") and inRange == 0 ) then
			if button.FB and button.FB.coloring ~= "usable_out_of_range" then
				icon:SetVertexColor(1.0, 0.1, 0.1);
				normalTexture:SetVertexColor(1.0, 0.1, 0.1);
				button.FB.coloring = "usable_out_of_range"
			end
		else
			if button.FB and button.FB.coloring ~= "usable_in_range" then
				icon:SetVertexColor(1.0, 1.0, 1.0);
				normalTexture:SetVertexColor(1.0, 1.0, 1.0);
				button.FB.coloring = "usable_in_range"
			end
		end
	elseif ( notEnoughMana ) then
		if button.FB and button.FB.coloring ~= "not_enough_mana" then
			icon:SetVertexColor(0.5, 0.5, 1.0);
			normalTexture:SetVertexColor(0.5, 0.5, 1.0);
			button.FB.coloring = "not_enough_mana"
		end
	else
		if button.FB and button.FB.coloring ~= "not_usable" then
			icon:SetVertexColor(0.4, 0.4, 0.4);
			normalTexture:SetVertexColor(1.0, 1.0, 1.0);
			button.FB.coloring = "not_usable"
		end
	end
-- I stayed out of the original code as much as possible.  Here is one place I had to
-- put code.  In order for user settings to override the color - it must go here.
	if button.FB and button.FB.State["icon"] then
		local bcolors = button.FB.State["icon"]
		icon:SetVertexColor(bcolors[1], bcolors[2], bcolors[3])
	end
	
-- Digital cooldown - set the hotkey text to '%c' to get a digital cooldown
	if button.FB and button.FB.State["hotkeytext"] then
		if button.FB.State["hotkeytext"] == "%c" then
			local start, duration, enable = GetActionCooldown(button:GetID());
			if start > 0 then
				count:SetText(format("%d",duration-(GetTime()-start)))
				count:SetVertexColor(1.0, 1.0, .5)
			else
				count:SetText("**")
				count:SetVertexColor(.5, 1.0, .5)
			end
		end
	end
this = button
end

function FlexBarButton_UpdateCount()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code - purpose unknown at this time
	local text = getglobal(button:GetName().."Count");
	local count = GetActionCount(FlexBarButton_GetID(button));
	if ( count > 1 ) then
		text:SetText(count);
	else
		text:SetText("");
	end
this = button
end

function FlexBarButton_UpdateCooldown()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code - looks like it updates the little cooldown spinner.
	local cooldown = getglobal(button:GetName().."Cooldown");
	local hotkey = getglobal(button:GetName().."HotKey");
	local start, duration, enable = GetActionCooldown(FlexBarButton_GetID(button));
	-- if digital cooldowns are enabled, disable spinner - it obscures the display.
	if button.FB and button.FB.State["hotkeytext"] and button.FB.State["hotkeytext"] ~= "%c" then
		CooldownFrame_SetTimer(cooldown, start, duration, enable);
	end
this = button
end

function FlexBarButton_OnEvent(event)
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Event code for individual buttons 
	if( event == "VARIABLES_LOADED" ) then
	end
	
	if( event == "PLAYER_ENTERING_WORLD" ) then
		local id = FlexBarButton_GetID(button);
		if( id >= 97 and id <= 108 ) then
			if( UnitClass("player") == "Druid" ) then
				button:SetID(id - 12);
			elseif( UnitClass("player") == "Warrior" ) then
				button:SetID(id - 36);
			end
		end
		this = button
		return;
	end

-- Blizzard code, unmodified to the best of my knowledge from here on down.
	if ( event == "ACTIONBAR_SLOT_CHANGED" ) then
		if ( arg1 == -1 or arg1 == FlexBarButton_GetID(button) ) then
			FlexBarButton_Update();
		end
		this = button
		return;
	end
	if ( event == "PLAYER_AURAS_CHANGED") then
		FlexBarButton_Update();
		FlexBarButton_UpdateState();
		this = button
		return;
	end
	if ( event == "ACTIONBAR_SHOWGRID" ) then
		FlexBarButton_ShowGrid();
		this = button
		return;
	end
	if ( event == "ACTIONBAR_HIDEGRID" ) then
		FlexBarButton_HideGrid();
		this = button
		return;
	end
	if ( event == "UPDATE_BINDINGS" ) then
		FlexBarButton_UpdateHotkeys();
	end

	-- All event handlers below this line MUST only be valid when the button is visible
	if ( not button:IsVisible() ) then
		this = button
		return;
	end

	if ( event == "PLAYER_TARGET_CHANGED" ) then
		FlexBarButton_UpdateUsable();
		this = button
		return;
	end
	if ( event == "UNIT_AURASTATE" ) then
		if ( arg1 == "player" or arg1 == "target" ) then
			FlexBarButton_UpdateUsable();
		end
		this = button
		return;
	end
	if ( event == "UNIT_INVENTORY_CHANGED" ) then
		if ( arg1 == "player" ) then
			FlexBarButton_Update();
		end
		this = button
		return;
	end
	if ( event == "ACTIONBAR_UPDATE_STATE" ) then
		FlexBarButton_UpdateState();
		this = button
		return;
	end
	if ( event == "ACTIONBAR_UPDATE_USABLE" or event == "UPDATE_INVENTORY_ALERTS" or event == "ACTIONBAR_UPDATE_COOLDOWN" ) then
		FlexBarButton_UpdateUsable();
		FlexBarButton_UpdateCooldown();
		this = button
		return;
	end
	if ( event == "CRAFT_SHOW" or event == "CRAFT_CLOSE" or event == "TRADE_SKILL_SHOW" or event == "TRADE_SKILL_CLOSE" ) then
		FlexBarButton_UpdateState();
		this = button
		return;
	end
	if ( arg1 == "player" and (event == "UNIT_HEALTH" or event == "UNIT_MANA" or event == "UNIT_RAGE" or event == "UNIT_FOCUS" or event == "UNIT_ENERGY") ) then
		FlexBarButton_UpdateUsable();
		this = button
		return;
	end
	if ( event == "PLAYER_ENTER_COMBAT" ) then
		IN_ATTACK_MODE = 1;
		if ( IsAttackAction(FlexBarButton_GetID(button)) ) then
			FlexBarButton_StartFlash();
		end
		this = button
		return;
	end
	if ( event == "PLAYER_LEAVE_COMBAT" ) then
		IN_ATTACK_MODE = 0;
		if ( IsAttackAction(FlexBarButton_GetID(button)) ) then
			FlexBarButton_StopFlash();
		end
		this = button
		return;
	end
	if ( event == "PLAYER_COMBO_POINTS" ) then
		FlexBarButton_UpdateUsable();
		this = button
		return;
	end
	if ( event == "START_AUTOREPEAT_SPELL" ) then
		IN_AUTOREPEAT_MODE = 1;
		if ( IsAutoRepeatAction(FlexBarButton_GetID(button)) ) then
			FlexBarButton_StartFlash();
		end
		this = button
		return;
	end
	if ( event == "STOP_AUTOREPEAT_SPELL" ) then
		IN_AUTOREPEAT_MODE = nil;
		if ( FlexBarButton_IsFlashing() and not IsAttackAction(FlexBarButton_GetID(button)) ) then
			FlexBarButton_StopFlash();
		end
		this = button
		return;
	end
this = button
end

function FlexBarButton_SetTooltip()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code
-- appears to reset the manually set this from my routines.
	GameTooltip_SetDefaultAnchor(GameTooltip, button)	
--	GameTooltip:SetOwner(this);
	if ( GameTooltip:SetAction(FlexBarButton_GetID(button)) ) then
		button.updateTooltip = TOOLTIP_UPDATE_TIME;
	else
		button.updateTooltip = nil;
	end
this = button
end


function FlexBarButton_OnUpdate(elapsed)
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code
	if ( FlexBarButton_IsFlashing() ) then
		button.flashtime = button.flashtime - elapsed;
		if ( button.flashtime <= 0 ) then
			local overtime = -button.flashtime;
			if ( overtime >= ATTACK_BUTTON_FLASH_TIME ) then
				overtime = 0;
			end
			button.flashtime = ATTACK_BUTTON_FLASH_TIME - overtime;

			local flashTexture = getglobal(button:GetName().."Flash");
			if ( flashTexture:IsVisible() ) then
				flashTexture:Hide();
			else
				flashTexture:Show();
			end
		end
	end
	-- Handle range indicator
	if ( button.rangeTimer ) then
		if ( button.rangeTimer < 0 ) then
			FlexBarButton_UpdateUsable();
			button.rangeTimer = TOOLTIP_UPDATE_TIME;
		else
			button.rangeTimer = button.rangeTimer - elapsed;
		end
	end

	if ( not button.updateTooltip ) then
		return;
	end

	button.updateTooltip = button.updateTooltip - elapsed;
	if ( button.updateTooltip > 0 ) then
		return;
	end

	if ( GameTooltip:IsOwned(button) ) then
		FlexBarButton_SetTooltip();
	else
		button.updateTooltip = nil;
	end
this = button
end

function FlexBarButton_GetID(button)
-- Telo's code
	if ( button == nil ) then
		message("nil button passed into FlexBarButton_GetID(), contact Jeff");
		return 0;
	end
	this = button
	return (button:GetID())
end

function FlexBarButton_StartFlash()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code
	button.flashing = 1;
	button.flashtime = 0;
	FlexBarButton_UpdateState();
this = button
end

function FlexBarButton_StopFlash()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code
	button.flashing = 0;
	getglobal(button:GetName().."Flash"):Hide();
	FlexBarButton_UpdateState();
this = button
end

function FlexBarButton_IsFlashing()
-- Desperate attempt to fix Dagar's Mac crash
if not string.find(this:GetName(),"FlexBarButton") then return end
local button = this
-- Blizzard code
	if ( button.flashing == 1 ) then
		return 1;
	else
		return nil;
	end
this = button
end

function FlexBar_LoadDefaults()
	util:Echo("Flex Bar - first use")
end

------------------------- Bindings ---------------------------------------
BINDING_HEADER_FLEXBAR = "FlexBar Buttons";
BINDING_NAME_FLEXACTIONBUTTON1 = "FlexBar Button 1";
BINDING_NAME_FLEXACTIONBUTTON2 = "FlexBar Button 2";
BINDING_NAME_FLEXACTIONBUTTON3 = "FlexBar Button 3";
BINDING_NAME_FLEXACTIONBUTTON4 = "FlexBar Button 4";
BINDING_NAME_FLEXACTIONBUTTON5 = "FlexBar Button 5";
BINDING_NAME_FLEXACTIONBUTTON6 = "FlexBar Button 6";
BINDING_NAME_FLEXACTIONBUTTON7 = "FlexBar Button 7";
BINDING_NAME_FLEXACTIONBUTTON8 = "FlexBar Button 8";
BINDING_NAME_FLEXACTIONBUTTON9 = "FlexBar Button 9";
BINDING_NAME_FLEXACTIONBUTTON10 = "FlexBar Button 10";
BINDING_NAME_FLEXACTIONBUTTON11 = "FlexBar Button 11";
BINDING_NAME_FLEXACTIONBUTTON12 = "FlexBar Button 12";
BINDING_NAME_FLEXACTIONBUTTON13 = "FlexBar Button 13";
BINDING_NAME_FLEXACTIONBUTTON14 = "FlexBar Button 14";
BINDING_NAME_FLEXACTIONBUTTON15 = "FlexBar Button 15";
BINDING_NAME_FLEXACTIONBUTTON16 = "FlexBar Button 16";
BINDING_NAME_FLEXACTIONBUTTON17 = "FlexBar Button 17";
BINDING_NAME_FLEXACTIONBUTTON18 = "FlexBar Button 18";
BINDING_NAME_FLEXACTIONBUTTON19 = "FlexBar Button 19";
BINDING_NAME_FLEXACTIONBUTTON20 = "FlexBar Button 20";
BINDING_NAME_FLEXACTIONBUTTON21 = "FlexBar Button 21";
BINDING_NAME_FLEXACTIONBUTTON22 = "FlexBar Button 22";
BINDING_NAME_FLEXACTIONBUTTON23 = "FlexBar Button 23";
BINDING_NAME_FLEXACTIONBUTTON24 = "FlexBar Button 24";
BINDING_NAME_FLEXACTIONBUTTON25 = "FlexBar Button 25";
BINDING_NAME_FLEXACTIONBUTTON26 = "FlexBar Button 26";
BINDING_NAME_FLEXACTIONBUTTON27 = "FlexBar Button 27";
BINDING_NAME_FLEXACTIONBUTTON28 = "FlexBar Button 28";
BINDING_NAME_FLEXACTIONBUTTON29 = "FlexBar Button 29";
BINDING_NAME_FLEXACTIONBUTTON30 = "FlexBar Button 30";
BINDING_NAME_FLEXACTIONBUTTON31 = "FlexBar Button 31";
BINDING_NAME_FLEXACTIONBUTTON32 = "FlexBar Button 32";
BINDING_NAME_FLEXACTIONBUTTON33 = "FlexBar Button 33";
BINDING_NAME_FLEXACTIONBUTTON34 = "FlexBar Button 34";
BINDING_NAME_FLEXACTIONBUTTON35 = "FlexBar Button 35";
BINDING_NAME_FLEXACTIONBUTTON36 = "FlexBar Button 36";
BINDING_NAME_FLEXACTIONBUTTON37 = "FlexBar Button 37";
BINDING_NAME_FLEXACTIONBUTTON38 = "FlexBar Button 38";
BINDING_NAME_FLEXACTIONBUTTON39 = "FlexBar Button 39";
BINDING_NAME_FLEXACTIONBUTTON40 = "FlexBar Button 40";
BINDING_NAME_FLEXACTIONBUTTON41 = "FlexBar Button 41";
BINDING_NAME_FLEXACTIONBUTTON42 = "FlexBar Button 42";
BINDING_NAME_FLEXACTIONBUTTON43 = "FlexBar Button 43";
BINDING_NAME_FLEXACTIONBUTTON44 = "FlexBar Button 44";
BINDING_NAME_FLEXACTIONBUTTON45 = "FlexBar Button 45";
BINDING_NAME_FLEXACTIONBUTTON46 = "FlexBar Button 46";
BINDING_NAME_FLEXACTIONBUTTON47 = "FlexBar Button 47";
BINDING_NAME_FLEXACTIONBUTTON48 = "FlexBar Button 48";
BINDING_NAME_FLEXACTIONBUTTON49 = "FlexBar Button 49";
BINDING_NAME_FLEXACTIONBUTTON50 = "FlexBar Button 50";
BINDING_NAME_FLEXACTIONBUTTON51 = "FlexBar Button 51";
BINDING_NAME_FLEXACTIONBUTTON52 = "FlexBar Button 52";
BINDING_NAME_FLEXACTIONBUTTON53 = "FlexBar Button 53";
BINDING_NAME_FLEXACTIONBUTTON54 = "FlexBar Button 54";
BINDING_NAME_FLEXACTIONBUTTON55 = "FlexBar Button 55";
BINDING_NAME_FLEXACTIONBUTTON56 = "FlexBar Button 56";
BINDING_NAME_FLEXACTIONBUTTON57 = "FlexBar Button 57";
BINDING_NAME_FLEXACTIONBUTTON58 = "FlexBar Button 58";
BINDING_NAME_FLEXACTIONBUTTON59 = "FlexBar Button 59";
BINDING_NAME_FLEXACTIONBUTTON60 = "FlexBar Button 60";
BINDING_NAME_FLEXACTIONBUTTON61 = "FlexBar Button 61";
BINDING_NAME_FLEXACTIONBUTTON62 = "FlexBar Button 62";
BINDING_NAME_FLEXACTIONBUTTON63 = "FlexBar Button 63";
BINDING_NAME_FLEXACTIONBUTTON64 = "FlexBar Button 64";
BINDING_NAME_FLEXACTIONBUTTON65 = "FlexBar Button 65";
BINDING_NAME_FLEXACTIONBUTTON66 = "FlexBar Button 66";
BINDING_NAME_FLEXACTIONBUTTON67 = "FlexBar Button 67";
BINDING_NAME_FLEXACTIONBUTTON68 = "FlexBar Button 68";
BINDING_NAME_FLEXACTIONBUTTON69 = "FlexBar Button 69";
BINDING_NAME_FLEXACTIONBUTTON70 = "FlexBar Button 70";
BINDING_NAME_FLEXACTIONBUTTON71 = "FlexBar Button 71";
BINDING_NAME_FLEXACTIONBUTTON72 = "FlexBar Button 72";
BINDING_NAME_FLEXACTIONBUTTON73 = "FlexBar Button 73";
BINDING_NAME_FLEXACTIONBUTTON74 = "FlexBar Button 74";
BINDING_NAME_FLEXACTIONBUTTON75 = "FlexBar Button 75";
BINDING_NAME_FLEXACTIONBUTTON76 = "FlexBar Button 76";
BINDING_NAME_FLEXACTIONBUTTON77 = "FlexBar Button 77";
BINDING_NAME_FLEXACTIONBUTTON78 = "FlexBar Button 78";
BINDING_NAME_FLEXACTIONBUTTON79 = "FlexBar Button 79";
BINDING_NAME_FLEXACTIONBUTTON80 = "FlexBar Button 80";
BINDING_NAME_FLEXACTIONBUTTON81 = "FlexBar Button 81";
BINDING_NAME_FLEXACTIONBUTTON82 = "FlexBar Button 82";
BINDING_NAME_FLEXACTIONBUTTON83 = "FlexBar Button 83";
BINDING_NAME_FLEXACTIONBUTTON84 = "FlexBar Button 84";
BINDING_NAME_FLEXACTIONBUTTON85 = "FlexBar Button 85";
BINDING_NAME_FLEXACTIONBUTTON86 = "FlexBar Button 86";
BINDING_NAME_FLEXACTIONBUTTON87 = "FlexBar Button 87";
BINDING_NAME_FLEXACTIONBUTTON88 = "FlexBar Button 88";
BINDING_NAME_FLEXACTIONBUTTON89 = "FlexBar Button 89";
BINDING_NAME_FLEXACTIONBUTTON90 = "FlexBar Button 90";
BINDING_NAME_FLEXACTIONBUTTON91 = "FlexBar Button 91";
BINDING_NAME_FLEXACTIONBUTTON92 = "FlexBar Button 92";
BINDING_NAME_FLEXACTIONBUTTON93 = "FlexBar Button 93";
BINDING_NAME_FLEXACTIONBUTTON94 = "FlexBar Button 94";
BINDING_NAME_FLEXACTIONBUTTON95 = "FlexBar Button 95";
BINDING_NAME_FLEXACTIONBUTTON96 = "FlexBar Button 96";
BINDING_HEADER_FLEXBAR_EVENTS = "FlexBar Events";
BINDING_NAME_FLEXBAREVENT1 = "FlexBar Event 1";
BINDING_NAME_FLEXBAREVENT2 = "FlexBar Event 2";
BINDING_NAME_FLEXBAREVENT3 = "FlexBar Event 3";
BINDING_NAME_FLEXBAREVENT4 = "FlexBar Event 4";
BINDING_NAME_FLEXBAREVENT5 = "FlexBar Event 5";
BINDING_NAME_FLEXBAREVENT6 = "FlexBar Event 6";
BINDING_NAME_FLEXBAREVENT7 = "FlexBar Event 7";
BINDING_NAME_FLEXBAREVENT8 = "FlexBar Event 8";
BINDING_NAME_FLEXBAREVENT9 = "FlexBar Event 9";
BINDING_NAME_FLEXBAREVENT10 = "FlexBar Event 10";
BINDING_NAME_FLEXBAREVENT11 = "FlexBar Event 11";
BINDING_NAME_FLEXBAREVENT12 = "FlexBar Event 12";
BINDING_NAME_FLEXBAREVENT13 = "FlexBar Event 13";
BINDING_NAME_FLEXBAREVENT14 = "FlexBar Event 14";
BINDING_NAME_FLEXBAREVENT15 = "FlexBar Event 15";
BINDING_NAME_FLEXBAREVENT16 = "FlexBar Event 16";
BINDING_NAME_FLEXBAREVENT17 = "FlexBar Event 17";
BINDING_NAME_FLEXBAREVENT18 = "FlexBar Event 18";
BINDING_NAME_FLEXBAREVENT19 = "FlexBar Event 19";
BINDING_NAME_FLEXBAREVENT20 = "FlexBar Event 20";
BINDING_NAME_FLEXBAREVENT21 = "FlexBar Event 21";
BINDING_NAME_FLEXBAREVENT22 = "FlexBar Event 22";
BINDING_NAME_FLEXBAREVENT23 = "FlexBar Event 23";
BINDING_NAME_FLEXBAREVENT24 = "FlexBar Event 24";
BINDING_NAME_FLEXBAREVENT25 = "FlexBar Event 25";
BINDING_NAME_FLEXBAREVENT26 = "FlexBar Event 26";
BINDING_NAME_FLEXBAREVENT27 = "FlexBar Event 27";
BINDING_NAME_FLEXBAREVENT28 = "FlexBar Event 28";
BINDING_NAME_FLEXBAREVENT29 = "FlexBar Event 29";
BINDING_NAME_FLEXBAREVENT30 = "FlexBar Event 30";
