
--[[
     Totem Circle
      Two-level mouse menu that contains all the shaman totems.
       * First level displays four buttons, one for each element,
          in a circle around the mouse cursor, triggered by the
          event 1 binding (i.e. middlemouse)
       * Second level displays the totems for a element when the
          apropriate first level button is moused over.
       * Menu disappears when a totem is selected, or event 1 is
          triggered again.
--]]
FB_C_TotemCircle = {

-- All buttons involved will be 80% scale
   "Scale button=47-72 scale=8",

--[[
     Create five button groups:
      One for the main menu circle
      Four to be used as the totem groups (one for each element)
--]]

-- Main totem ring
   "MoveRel button=48 trgbtn=47 dX=32 dY=32",
   "MoveRel button=49 trgbtn=47 dX=66 dY=0",
   "MoveRel button=50 trgbtn=47 dX=32 dY=-32",
   "Group button=47-50 anchor=47",

-- Earth has five totems total
   "Group button=51-55 anchor=51",
   "VerticalGroup group=51 width=1 padding=2",

-- Fire has five totems total
   "Group button=56-60 anchor=56",
   "VerticalGroup group=56 width=1 padding=2",

-- Air has six totems total
   "Group button=61-66 anchor=61",
   "HorizontalGroup group=61 height=1 padding=2",

-- Water has six totems total (including optional Mana Tide)
   "Group button=67-72 anchor=67",
   "HorizontalGroup group=67 height=1 padding=2",

--[[
     Triggering the menu via event 1 causes:
      1. Move the circle menu to be centered around the mouse cursor
      2. Start all menu selections at 80% opacity
      3. Show/hide the circle menu buttons
      4. Hide the totem groups
--]]
   "MoveToMouse button=47 dX=-40 dY=12 on='BindingKeyDown' target=1",
   "Fade group=47 alpha=8              on='BindingKeyDown' target=1",
   "Show group=47 toggle='true'        on='BindingKeyDown' target=1",
   "Hide button=51-72                  on='BindingKeyDown' target=1",

--[[
     When mousing over a totem type in the circle menu:
       1. make the active button fully opaque
       2. fade the other buttons in the circle menu
       3. move the selected group into position
       4. show the totem group for that button
       5. hide the other totem groups
--]]
-- 9 o'clock, earth, buttons vertical to the left of the circle
   "Fade button=[47         ] alpha=10       on='MouseEnterButton' target=47",
   "Fade button=[   48 49 50] alpha=4        on='MouseEnterButton' target=47",
   "MoveRel button=51 trgbtn=47 dX=-32 dY=59 on='MouseEnterButton' target=47",
   "Show button=51-55                        on='MouseEnterButton' target=47",
   "Hide button=56-72                        on='MouseEnterButton' target=47",

-- 12 o'clock, air, buttons horizontal above the circle
   "Fade button=[   48      ] alpha=10       on='MouseEnterButton' target=48",
   "Fade button=[47    49 50] alpha=4        on='MouseEnterButton' target=48",
   "MoveRel button=61 trgbtn=48 dX=-72 dY=32 on='MouseEnterButton' target=48",
   "Show button=61-66                        on='MouseEnterButton' target=48",
   "Hide button=[51 52 53 54 55 56 57 58 59 60 67 68 69 70 71 72] on='MouseEnterButton' target=48",

-- 3 o'clock, fire, buttons vertical to the right of the circle
   "Fade button=[      49   ] alpha=10       on='MouseEnterButton' target=49",
   "Fade button=[47 48    50] alpha=4        on='MouseEnterButton' target=49",
   "MoveRel button=56 trgbtn=49 dX=32 dY=59  on='MouseEnterButton' target=49",
   "Show button=56-60                        on='MouseEnterButton' target=49",
   "Hide button=[51 52 53 54 55 61 62 63 64 65 66 67 68 69 70 71 72] on='MouseEnterButton' target=49",

-- 6 o'clock, water, buttons horizontal below the circle
   "Fade button=[         50] alpha=10        on='MouseEnterButton' target=50",
   "Fade button=[47 48 49   ] alpha=4         on='MouseEnterButton' target=50",
   "MoveRel button=67 trgbtn=50 dX=-72 dY=-32 on='MouseEnterButton' target=50",
   "Show button=67-72                         on='MouseEnterButton' target=50",
   "Hide button=51-66                         on='MouseEnterButton' target=50",


-- Hide all once a totem is selected
   "Hide button=47-72 on='LeftButtonClick' target=47-72",
   "Lock button=47-72",
} 