--[[
	Non Event functions for FlexBar
	Last Modified
		01/09/2005	Initial version
--]]

local util = Utility_Class:New()
-- Var to hold last target reaction, can be local as it's only called in this module
-- all the following can be local as they are only referenced here.
local FBLastTarget
-- var to hold last form for shape/stance/aura/stealth etc changes
local FBLastform = "none"
-- var to hold last pet name for pet existence check.
local FBPetname
-- var to hold group member existence info for  existence check
local FBGroupmates = {}

function FB_ApplySettings()
-- For each button, apply the saved settings. 
-- Originally every slash command came through here.
-- changed that so that any conditional slash commands
-- call the appropriate apply function only for the buttons
-- they are changing, and save only the setting they are
-- changing to the profile.  The reason being I don't want
-- all the potentially expensive code in scaling etc to execute
-- on conditionals if they don't have to.
local index
	for index = 1,FBNumButtons do
		local button, frame = FB_GetWidgets(index)
		-- Apply hide state
		FB_ApplyHidden(button, frame)
		-- Apply alpha
		FB_ApplyAlpha(button)
		-- Apply scale
		FB_ApplyScale(button)
		-- Apply lock state
		FB_ApplyLocked(button, frame)
		-- Update hotkey text
		local saved = this
		this = button
		FlexBarButton_UpdateHotkeys()
		this = saved
		-- Apply remap
		FB_ApplyRemap(button)
		-- Apply show grid
		FB_ApplyGrid(button)
		-- Get coordinates (of the bottom right corner of the handle)
		FB_GetCoords(button, frame)
		-- Save button state to character profile
		FB_SaveState(button)
	end
end

function FB_ResyncSettings()
-- For each button, get coords and save to profile. 
-- Coords are only saved by wow for user placed items - the others
-- don't get saved.
local index
	for index = 1, FBNumButtons do
		local button, frame = FB_GetWidgets(index)
		-- Get coordinates (of the bottom right corner of the handle)
		FB_GetCoords(button, frame)
		-- Save button coords to character profile
		FBSavedProfile[UnitName("player")][index].State["xcoord"] = button.FB.State["xcoord"]
		FBSavedProfile[UnitName("player")][index].State["ycoord"] = button.FB.State["ycoord"]
	end
	-- also check to see if the scale has gotten munged by an alt tab, and reapply settings
	if FlexBar:GetScale() > .25 then
		FlexBar:SetScale(.25)
		FB_ApplySettings()
	end
end

function FB_ReformGroups()
-- For each button, check group status, attach to appropriate frame. 
local index
	for index = 1,FBNumButtons do
		local button, frame = FB_GetWidgets(index)
		if button.FB.State["group"] and (button.FB.State["group"] ~= index) then
			FB_LockButton(frame)
			local anchor = FB_GetWidgets(button.FB.State["group"])
			local x = button.FB.State["xcoord"] - anchor.FB.State["xcoord"]
			local y = button.FB.State["ycoord"] - anchor.FB.State["ycoord"]
			frame:ClearAllPoints()
			frame:SetPoint("BOTTOMRIGHT", "FlexFrame" .. button.FB.State["group"], "BOTTOMRIGHT", x, y)
			button.FB.State["locked"] = true
		elseif button.FB.State["group"] and (button.FB.State["group"] == index) then
			local x = button.FB.State["xcoord"]
			local y = button.FB.State["ycoord"]
			frame:ClearAllPoints()
			frame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", x, y)
		else
			local x = button.FB.State["xcoord"]
			local y = button.FB.State["ycoord"]
			frame:ClearAllPoints()
			frame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", x, y)
		end
	end
	FB_ApplySettings()
	-- Generate group data for use in InGroupBounds()
	for index = 1, FBNumButtons do
		local button, frame = FB_GetWidgets(index)
		if button.FB.State["group"] == index then
			FBGroupData[index] = FB_GetBoundingButtons(index)
		else
			FBGroupData[index] = nil;
		end
	end
end

function FB_RaiseEvent(event, source)
-- raise no events until a profile is loaded.
if not FBProfileLoaded then return end
-- Respond to events raised throughout the code.
-- Source is the originator of the event
if FBDisplayEvent then
	util:Print(event)
	util:Print(source)
end
event = string.lower(event)
if type(source) == "string" then source = string.lower(source) end
local index
	for index = 1,FBEvents.n do
		if FBEvents[index] and string.lower(FBEvents[index]["on"]) == event then
			if 	not FBEvents[index]["target"] or 
				FB_InTable(source, FBEvents[index]["target"]) then
				FBcmd:Dispatch(FBEvents[index]["command"])
			end
		end
	end
end

-- Group utility functions
function FB_CheckGroups()
-- Check for mouse entering/leaving groups
	local group
	local x,y = GetCursorPosition()
	x = x / UIParent:GetScale()
	y = y / UIParent:GetScale()
	if abs(x-FBCursorLoc.x) < 1 and abs(y-FBCursorLoc.y) < 1 then return end
	for group = 1, FBNumButtons do
		if FBGroupData[group] then 
			if 	FB_InGroupBounds(group, x, y) and
				not FB_InGroupBounds(group, FBCursorLoc.x, FBCursorLoc.y) then
				FB_RaiseEvent("MouseEnterGroup", group)
			end
			if 	not FB_InGroupBounds(group, x, y) and
				FB_InGroupBounds(group, FBCursorLoc.x, FBCursorLoc.y) then
				FB_RaiseEvent("MouseLeaveGroup", group)
			end
		end
	end
	FBCursorLoc.x = x
	FBCursorLoc.y = y
end

function FB_InGroupBounds(group, x, y)
-- Check to see if x,y is inside group
local button, frame = FB_GetWidgets(group)
local dim = 36 * button:GetScale()
	if 	x < FBGroupData[group]["left"].FB.State["xcoord"] - 1 or
		x > FBGroupData[group]["right"].FB.State["xcoord"] + dim or
		y < FBGroupData[group]["bottom"].FB.State["ycoord"] - dim or
		y > FBGroupData[group]["top"].FB.State["ycoord"] + 1 then
		return false
	else
		return true
	end
end

function FB_GetBoundingButtons(group)
-- Return a table with the bounding buttons of a group, to
-- speed up in bounds checks.
local	members = FB_GetGroupMembers(group)
local 	index
local 	bounds = {}
local 	button1, frame = FB_GetWidgets(members[1])
		bounds["top"] = button1
		bounds["bottom"] = button1
		bounds["left"] = button1
		bounds["right"] = button1
	for index = 1, members.n do
		local button, frame = FB_GetWidgets(members[index])
		if button.FB.State["xcoord"] > bounds["right"].FB.State["xcoord"] then
			bounds["right"] = button
		end
		if button.FB.State["xcoord"] < bounds["left"].FB.State["xcoord"] then
			bounds["left"] = button
		end
		if button.FB.State["ycoord"] > bounds["top"].FB.State["ycoord"] then
			bounds["top"] = button
		end
		if button.FB.State["ycoord"] < bounds["bottom"].FB.State["ycoord"] then
			bounds["bottom"] = button
		end
	end
	return bounds
end

function FB_GetGroupMembers(group)
-- returns a table with all the members in <group>
local 	members = {}
		members.n = 0
local 	index
	for index = 1, FBNumButtons do
		local button, frame = FB_GetWidgets(index)
		if button.FB.State["group"] == group then
			members.n = members.n + 1
			members[members.n] = index
		end
	end
	return members
end

function FB_VerticalGroup(group, width, padding)
-- Arranges buttons in group vertically, width columns wide with
-- the specified padding between them.
	local members = FB_GetGroupMembers(group)
	if members.n < 1 then return end
	local index = 1
	local button, frame = FB_GetWidgets(members[index])
	local dim = padding + (38 * (button.FB.State["scale"] or 1))
--	dim = dim / UIParent:GetScale()
	while index <= members.n do
		local index2
		for index2 = 1, width do
			if index <= members.n then
				local button, frame = FB_GetWidgets(members[index])
				FB_MoveButtonABS(button, 300 + ((index2-1)*dim), 500-(floor((index-1)/width)*dim))
			end
			index = index+1
		end
	end
	FB_ReformGroups()
end
			
function FB_HorizontalGroup(group, height, padding)
-- Arranges buttons in group horizontally, height rows high with
-- the specified padding between them.
	local members = FB_GetGroupMembers(group)
	if members.n < 1 then return end
	local index = 1
	local button, frame = FB_GetWidgets(members[index])
	local dim = padding + (38 * (button.FB.State["scale"] or 1))
--	dim = dim / UIParent:GetScale()
	while index <= members.n do
		local index2
		for index2 = 1, height do
			if index <= members.n then
				local button, frame = FB_GetWidgets(members[index])
				FB_MoveButtonABS(button, 300+(floor((index-1)/height)*dim), 500-((index2-1)*dim))
			end
			index = index+1
		end
	end
	FB_ReformGroups()
end
			
function FB_CircularGroup(group, padding)
-- Arranges buttons in group in a circle with the specified padding between them.
-- Starts at 10 O'clock and moves clockwise, with the 7th in the center.
-- Max buttons for this is 7
	local coords = {{-1,.5},{0,1},{1,.5},{1,-.5},{0,-1},{-1,-.5},{0,0}}
	local members = FB_GetGroupMembers(group)
	if members.n < 1 or members.n > 7 then return end
	local index = 1
	local button, frame = FB_GetWidgets(members[index])
	local dim = padding + (38 * (button.FB.State["scale"] or 1))
--	dim = dim / UIParent:GetScale()
	for index = 1, members.n do
		local button, frame = FB_GetWidgets(members[index])
		FB_MoveButtonABS(button, 300+(coords[index][1]*dim), 300+(coords[index][2]*dim))
	end
	FB_ReformGroups()
end
			
	
-- Button utility functions
	
function FB_GetWidgets(buttonnum)
-- returns the button and frame (handle) associated with buttonnum
local	button	= getglobal("FlexBarButton" .. buttonnum)
local	frame	= getglobal("FlexFrame" .. buttonnum)
	return button, frame
end

function FB_GetButtonNum(frame)
-- takes either a button or handle (frame) and returns the button number
local firsti, lasti, num = string.find(frame:GetName(), "(%d+)")
	return num+0
end

function FB_GetCoords(button, frame)
-- Save the coordinates of the handle - On programmatically moved buttons,
-- WoW won't do it automatically for us across sessions.
	button.FB.State["xcoord"] = frame:GetRight()
	button.FB.State["ycoord"] = frame:GetBottom()
end

function FB_SaveState(button)
-- Save the current button state to the current character's profile.
local firsti, lasti, buttonnum = string.find(button:GetName(), "(%d+)")
	if FBSavedProfile[UnitName("player")] == nil then
		FBSavedProfile[UnitName("player")] = {}
	end
	FBSavedProfile[UnitName("player")][buttonnum+0] = util:TableCopy(button.FB)
end

function FB_LoadState(button)
-- load the current button state from the current character's profile.
local firsti, lasti, buttonnum = string.find(button:GetName(), "(%d+)")
	if FBSavedProfile[UnitName("player")] == nil then
		return
	end
	button.FB = util:TableCopy(FBSavedProfile[UnitName("player")][buttonnum+0])
	button.FB.State["oldid"] = button:GetID()
	-- Move button to its saved location
	if button.FB.State["xcoord"] and button.FB.State["ycoord"] then
		local x = button.FB.State["xcoord"]
		local y = button.FB.State["ycoord"]
		local frame = getglobal("FlexFrame"..buttonnum)
		frame:ClearAllPoints()
		frame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", x, y)
	end
end

function FB_MoveButtonABS(button1, x, y)
-- Moves the button and handle so the top left of the button and bottom right of the
-- frame are x pixels to the right of the bottom right corner and y pixels above it.
-- will work with either a button or handle passed to it.
local buttonnum = FB_GetButtonNum(button1)
local button, frame = FB_GetWidgets(buttonnum)
	frame:ClearAllPoints()
	frame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", x, y)
	FB_GetCoords(button, frame)
end

function FB_MoveButtonRel(button1, button2, dx, dy)
-- Moves button1's top left point (bottom right of handle) x pixels to the left of
-- and y pixels above button2's top left point (bottom right of it's handle)
-- Leaves button anchored on UIParent.
local x = button2.FB.State["xcoord"] + dx
local y = button2.FB.State["ycoord"] + dy
local buttonnum = FB_GetButtonNum(button1)
local button, frame = FB_GetWidgets(buttonnum)
	frame:ClearAllPoints()
	frame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", x, y)
	FB_GetCoords(button, frame)
end

function FB_LockButton(frame)
-- Lock the button by shrinking the handle underneath the
-- corner of the button.
	if frame:GetHeight() == 1 then return end -- is already locked
	-- If it's user placed, the relative points change.  Reset it so we know that
	-- it is set with it's bottom-right corner relative to the UIParent's bottom left.
	if frame:IsUserPlaced() then
		local x = frame:GetRight()
		local y = frame:GetBottom()
		frame:ClearAllPoints()
		frame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", x, y)
	end
	frame:SetHeight(1)
	frame:SetWidth(1)
end

function FB_UnlockButton(frame)
-- unlock button by enlarging the handle to normal size.
	if frame:GetHeight() == 10 then return end -- is already unlocked
	-- If it's user placed, the relative points change.  Reset it so we know that
	-- it is set with it's bottom-right corner relative to the UIParent's bottom left.
	-- While I can't see how a button could be user placed if it's already locked,
	-- better safe than sorry.
	if frame:IsUserPlaced() then
		local x = frame:GetRight()
		local y = frame:GetBottom()
		frame:ClearAllPoints()
		frame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", x, y)
	end
	frame:SetHeight(10)
	frame:SetWidth(10)
end

-- Functions called by timers to check status of a variety of things and raise the appropriate events

function FB_GroupEvents()
-- Called every .5 seconds to check for a change in party status
local index
	for index = 1, 4 do
		local name = UnitName("party" .. index)
		if name and not FBGroupmates[index] then
			FB_RaiseEvent("GainPartymate", "party" .. index)
		end
		if not name and FBGroupmates[index] then
			FB_RaiseEvent("LosePartyMate", "party" .. index)
		end
		
		FBGroupmates[index] = name
	end
end

function FB_PetEvents()
-- Called every .5 seconds to check for a change in pet status
local petname = UnitName("pet")
	if petname and not FBPetname then
		FB_RaiseEvent("GainPet")
	end
	if not petname and FBPetname then
		FB_RaiseEvent("LosePet")
	end
	
	FBPetname = petname
end

function FB_FormEvents()
-- Called every .5 seconds to check for a change in form
local form = "none"
	if FBLastform then
		local index
		for index = 1, GetNumShapeshiftForms() do
			local _, name, active = GetShapeshiftFormInfo(index)
			if active then
				form = name
				break
			end
		end
	end
	if FBLastform ~= form then
		if FBLastform ~= "none" then
			FB_RaiseEvent("LoseAura", FBLastform)
		end
		if form ~= "none" then
			FB_RaiseEvent("GainAura", form)
		end
	end
	FBLastform = form
end

function FB_TargetEvents()
-- Called every .1 seconds to check for a target and whether it's friendly etc
	local reaction = {"","hostile","","neutral","friendly"}
	if UnitName("target") and not FBLastTarget then
	-- gained target
		FB_RaiseEvent("GainTarget", reaction[UnitReaction("player", "target")])
	elseif UnitName("target") and FBLastTarget ~= UnitReaction("player", "target") then
	-- target reaction changed
		FB_RaiseEvent("TargetReactionChanged" , reaction[UnitReaction("player", "target")])
	elseif not UnitName("target") and FBLastTarget then
	-- lost target
		FB_RaiseEvent("LostTarget")
	end
	FBLastTarget = UnitReaction("player", "target")
end

function FB_ManaEvents(button)
-- mana events.
	local index
	for index = 1, FBNumButtons do
		local button, frame = FB_GetWidgets(index)
		if HasAction(button:GetID()) then
			local isUsable, notEnoughMana = IsUsableAction(button:GetID());
			if button.FB then
				if button.FB.notEnoughMana and not notEnoughMana then
					FB_RaiseEvent("EnoughMana", FB_GetButtonNum(button))
				end
				if not button.FB.notEnoughMana and notEnoughMana then
					FB_RaiseEvent("NotEnoughMana", FB_GetButtonNum(button))
				end
				button.FB.notEnoughMana=notEnoughMana

				if button.FB.isUsable and not isUsable then
					FB_RaiseEvent("NotUsable", FB_GetButtonNum(button))
				end
				if not button.FB.isUsable and isUsable then
					FB_RaiseEvent("IsUsable", FB_GetButtonNum(button))
				end
				button.FB.isUsable = isUsable
				button.FB.notEnoughMana=notEnoughMana
			end
		end
	end
end

function FB_RangeEvents(button)
-- Range events.
	local index
	for index = 1, FBNumButtons do
		local button, frame = FB_GetWidgets(index)
		if HasAction(button:GetID()) then
			local inRange = IsActionInRange(button:GetID());
			if button.FB then
				if button.FB.inRange ~= 0 and inRange == 0 then
					FB_RaiseEvent("OutOfRange", FB_GetButtonNum(button))
				end
				if button.FB.inRange == 0 and inRange ~= 0 then
					FB_RaiseEvent("NowInRange", FB_GetButtonNum(button))
				end
				button.FB.inRange = inRange
			end
		end
	end
end

function FB_CooldownEvents(button)
-- raise cooldown events	
	local index
	for index = 1, FBNumButtons do
		local button, frame = FB_GetWidgets(index)
		if HasAction(button:GetID()) then
			if button.FB then
				local start, duration, enable = GetActionCooldown(button:GetID())
				if button.FB.start == nil then button.FB.start = 0 end
				if start == 0 and button.FB.start ~= 0 then
					FB_RaiseEvent("CoolDownMet", FB_GetButtonNum(button))
				end
				if start ~= 0 and button.FB.start == 0 then
					FB_RaiseEvent("CoolDownStart", FB_GetButtonNum(button))
				end
				button.FB.start = start
			end
		end
	end
end	

-- General Utility functions

function FB_Report(msg)
-- little helper function to display feedback of actions to the user.
if not FBVerbose then return end
	util:Print("FB> " .. msg)
end

function FB_LoadProfile()
-- Load the current character's profile back into the buttons.
if 	(UnitName("player") == nil) or
	(string.lower(UnitName("player")) == "unknown entity") then
	FBTimers["loadprofile"] = Timer_Class:New(1, nil, 
		".... loading", nil, FB_LoadProfile)
end
	
local index
	if type(FBSavedProfile[UnitName("player")]) ~= "table" then
		FBSavedProfile[UnitName("player")] = {}
		for index = 1, FBNumButtons do
			FBSavedProfile[UnitName("player")][index] = {}
			FBSavedProfile[UnitName("player")][index].State = {}
			FBSavedProfile[UnitName("player")][index].State["hidden"] = true
		end
		FBSavedProfile[UnitName("player")].Events = {}
		FBSavedProfile[UnitName("player")].Events.n = 0
	end
	for index = 1, FBNumButtons do
		local button, frame = FB_GetWidgets(index)
		FB_LoadState(button)
	end
	FB_ApplySettings()
	FBEvents = util:TableCopy(FBSavedProfile[UnitName("player")].Events)
	FB_ReformGroups()
	-- Using FlexBar's scale to see if the buttons need rescaling
	FlexBar:SetScale(.25)
	util:Echo("Flex Bar settings for " .. UnitName("player") .. " loaded")
	-- set a recurring timer to resync coords and save to profile
	FBTimers["resync"] = Timer_Class:New(2, true, nil, nil, FB_ResyncSettings)
	-- set a recurring timers to check for the variety of events that don't necessarily have real
	-- WoW events (some of these do, but was just easier to copy/paste the timer code.  A liittle
	-- sloppy, but the extra overhead for group checking is minimal, and for form checking not only
	-- minimal but well worth it to get fades that normally don't get reported in UNIT_LOSE_AURA)
	FBTimers["boundcheck"] = Timer_Class:New(.1, true, nil, nil, FB_CheckGroups)
	FBTimers["cooldowncheck"] = Timer_Class:New(1, true, nil, nil, FB_CooldownEvents)
	FBTimers["manacheck"] = Timer_Class:New(1, true, nil, nil, FB_ManaEvents)
	FBTimers["rangecheck"] = Timer_Class:New(.25, true, nil, nil, FB_RangeEvents)
	FBTimers["targetcheck"] = Timer_Class:New(.1, true, nil, nil, FB_TargetEvents)
	FBTimers["formcheck"] = Timer_Class:New(.5, true, nil, nil, FB_FormEvents)
	FBTimers["petcheck"] = Timer_Class:New(.5, true, nil, nil, FB_PetEvents)
	FBTimers["groupcheck"] = Timer_Class:New(.5, true, nil, nil, FB_GroupEvents)
	FBProfileLoaded=true;
end

function FB_GetButtonList(list)
-- regularize arguments to functions that take button lists
-- returns a list of buttons to be affected as a list, with the
-- number of buttons in list.n
local 	return_value = {}
		return_value.n = 0
	if list == nil then
		return return_value		
	elseif type(list) == "number" then
		return_value[1]=list
		return_value.n = 1
		return return_value
	elseif type(list) == "table" then
		local index
		list.n = 0
		for index in ipairs(list) do
			list.n = list.n+1
		end
		return_value=list
		return return_value
	else
		util:Print("Nil passed to FB_GetButtonList - report this")
	end
end

function FB_InTable(value1, table1)
-- Simple utility function to check for the existence of
-- value1 in table1
local index, value
	for index,value in pairs(table1) do
		if type(value) == "string" then value=string.lower(value) end
		if value == value1 then
			return true;
		end
	end
	return false
end

function FB_GetEventMsg(action, args)
-- Creates a new table with the parameters needed for RaiseEvent and populates FBEvents.
	local 	event = {}
-- if there is no "ON" directive then return nil
	if args["on"] == nil then  
		return nil 
	else
		event["on"] = args["on"]
	end
-- copy the target over to the event
	if args["target"] and type(args["target"]) == "table" then
		event["target"] = util:TableCopy(args["target"])
	else
		if args["target"] then
			event["target"] = {args["target"]}
		else
			event["target"] = nil
		end
	end
-- Set up the message string to go to Command_Class:Dispatch
	local	msg = action
	local	index, value
	for index, value in pairs(args) do
		if index~= "on" and index~="target" then
			if type(value) ~= "table" then
				if type(value) == "string" then value="'"..value.."'" end
				msg = msg .. " " .. index .. "=" .. value
			else
				msg = msg .. " " .. index .. "=" .. "["
				local index2, value2
				for index2, value2 in ipairs(value) do
					if type(value2) == "string" then value2="'"..value2.."'" end
					msg = msg .. " " .. value2
				end
				msg = msg .. " ]"
			end
		end
	end
-- Copy out the target text for the list events command.
	event["command"]=msg
	local targettext = "Target="
	if args["target"] == nil then
		targettext = targettext .. "No target"
	elseif type(args["target"]) == "string" then
		targettext = targettext .. "'" .. args["target"] .. "'"
	elseif type(args["target"]) == "table" then
		local index2, value2
		targettext=targettext .. "[ "
		for index2, value2 in ipairs(args["target"]) do
			targettext = targettext .. " " .. value2
		end
		targettext = targettext .. " ]"
	else
		targettext = targettext .. args["target"]
	end
	event["targettext"]=targettext

	FBEvents.n = FBEvents.n + 1
	FBEvents[FBEvents.n] = util:TableCopy(event)
	FB_Report("On " .. FBEvents[FBEvents.n]["on"] .. " " .. FBEvents[FBEvents.n]["targettext"] .. " : " .. FBEvents[FBEvents.n]["command"])
	FBSavedProfile[UnitName("player")].Events = util:TableCopy(FBEvents)
	return true
end

-- Apply saved states (From FB.State) to the buttons. 
-- Removed group checks - if the user wants an entire
-- group to change, they need to explicitly say so.

function FB_ApplyHidden(button, frame)
-- Apply hidden state
	if button.FB.State["hidden"] then
		frame:Hide()
	else
		frame:Show()
	end
end

function FB_ApplyAlpha(button)
-- Apply alpha
	if button.FB.State["alpha"] then
		button:SetAlpha(button.FB.State["alpha"])
	end
end

function FB_ApplyScale(button)
-- Apply scale
	if button.FB.State["scale"] then
		button:SetScale(button.FB.State["scale"] * UIParent:GetScale())
	else
		button:SetScale(UIParent:GetScale())
	end
end

function FB_ApplyLocked(button, frame)
-- Determine whether the button should be locked or not and apply
-- Only the anchor of a group may be locked/unlocked.
	if 	button.FB.State["group"] and 
		FB_GetButtonNum(button) ~= button.FB.State["group"] then 
		return 
	end;
	
	if button.FB.State["locked"] then
		FB_LockButton(frame)
	else
		FB_UnlockButton(frame)
	end
end

function FB_ApplyRemap(button)
-- Apply remap 
	if button.FB.State["remap"] then
		button:SetID(button.FB.State["remap"])
		local saved = this
		this = button
		FlexBarButton_Update()
		this = button
		FlexBarButton_UpdateState()
		this = saved
	end
end

function FB_ApplyGrid(button)
-- Apply grid
	if button.FB.State["hidegrid"] then
		button.showgrid=0
	else
		button.showgrid=1
		button:Show()
	end
	local saved = this
	this = button
	FlexBarButton_Update()
	this = button
	FlexBarButton_UpdateState()
	this = saved
end

