--[[
	Functions to implement slash commands for FlexBar
	Last Modified
		12/26/2004	Initial version
--]]

local util = Utility_Class:New()
local nextcommand = 0;
local currentconfig = nil
local REQUIRED = true
local STRICT = true
local OPTIONAL = false
local NON_STRICT = false
local NO_DEFAULT = nil;

function FB_Command_AddCommands(scmd)
-- Populate the command object, scmd, with the commands we want available
-- args are command, function, group, usage
	scmd:AddCommand("safeload", FB_Command_SafeLoad, "misc",
		"/Flexbar SafeLoad State=<'on' | 'off'>")
		scmd:AddParam("safeload","state",REQUIRED,STRICT,{"string"},{"on","On","off","Off"},NO_DEFAULT)
		
	scmd:AddCommand("loadprofile", FB_Command_LoadProfile, "misc",
		"/Flexbar LoadProfile Profile='profile'")
		scmd:AddParam("loadprofile","profile",REQUIRED,NON_STRICT,{"string"},nil,NO_DEFAULT)
		
	scmd:AddCommand("saveprofile", FB_Command_SaveProfile, "misc",
		"/Flexbar SaveProfile Profile='profile'")
		scmd:AddParam("saveprofile","profile",REQUIRED,NON_STRICT,{"string"},nil,NO_DEFAULT)
		
	scmd:AddCommand("remap", FB_Command_ReMap, "misc",
		"/FlexBar ReMap Button=<buttons> Base=<id> [Reset='True']")
		-- populate the parameter list for this command
		-- arguments are: command, parameter name, required, strict, types, allowed, default
		scmd:AddParam("remap","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("remap","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("remap","base"  ,REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=120}, nil)
		scmd:AddParam("remap","toggle",OPTIONAL,STRICT,{"string"},{"true"},nil)
		scmd:AddParam("remap","reset" ,OPTIONAL,NON_STRICT,{"string"},{"true"},nil)
		scmd:AddParam("remap","on"    ,OPTIONAL,NON_STRICT,{"string"},FBEventList,nil)
		scmd:AddParam("remap","target",OPTIONAL,NON_STRICT,nil,nil,nil)
		scmd:AddParam("remap","if"    ,OPTIONAL,NON_STRICT,{"string"},FBConditionList,nil)
		scmd:AddParam("remap","iftarg",OPTIONAL,NON_STRICT,nil,nil,nil)
		
	scmd:AddCommand("advanced", FB_Command_Advanced, "misc",
		"/FlexBar Advanced Button=<buttons> state=<'on' | 'off'>")
		-- populate the parameter list for this command
		-- arguments are: command, parameter name, required, strict, types, allowed, default
		scmd:AddParam("advanced","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("advanced","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("advanced","state" ,REQUIRED,NON_STRICT,{"string"},{"on","On","off","Off"}, nil)

	scmd:AddCommand("use", FB_Command_Use, "misc",
		"/FlexBar Use Button=<buttons>")
		-- populate the parameter list for this command
		-- arguments are: command, parameter name, required, strict, types, allowed, default

	scmd:AddCommand("resetall", FB_Command_ResetAll, "misc",
		"/FlexBar ResetAll Confirm='yes'")
		-- populate the parameter list for this command
		-- arguments are: command, parameter name, required, strict, types, allowed, default
		scmd:AddParam("resetall","confirm",REQUIRED,STRICT,{"string"},{"yes","Yes"}, nil)

	scmd:AddCommand("loadconfig", FB_Command_LoadConfig, "misc",
		"/FlexBar LoadConfig Config='ConfigName'")
		-- populate the parameter list for this command
		-- arguments are: command, parameter name, required, strict, types, allowed, default
		scmd:AddParam("loadconfig","config",REQUIRED,NON_STRICT,{"string"},nil, nil)

	scmd:AddCommand("hidegrid", FB_Command_HideGrid, "misc",
		"/FlexBar HideGrid Button=<buttons>")
		scmd:AddParam("hidegrid","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("hidegrid","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("hidegrid","toggle",OPTIONAL,STRICT,{"string"},{"true"},nil)

	scmd:AddCommand("showgrid", FB_Command_ShowGrid, "misc",
		"/FlexBar ShowGrid Button=<buttons>")
		scmd:AddParam("showgrid","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("showgrid","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("showgrid","toggle",OPTIONAL,STRICT,{"string"},{"true"},nil)

	scmd:AddCommand("lockicon", FB_Command_LockIcon, "misc",
		"/FlexBar LockIcon Button=<buttons> [Off='true']")
		scmd:AddParam("lockicon","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("lockicon","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("lockicon","off"   ,OPTIONAL,STRICT,{"string"},{"true", "True"},nil)

	scmd:AddCommand("text", FB_Command_Text, "misc",
		"/FlexBar Text Button=<buttons> Text=<text>")
		scmd:AddParam("text","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("text","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("text","text"  ,OPTIONAL,NON_STRICT,{"string"},nil,nil)

	scmd:AddCommand("verbose", FB_Command_Verbose, "misc",
		"/FlexBar Verbose State=<'on' | 'off'>")
		scmd:AddParam("verbose","state",REQUIRED,STRICT,{"string"},{"on","On","Off","off"},"on")
		
	scmd:AddCommand("tooltip", FB_Command_Tooltip, "misc",
		"/FlexBar Tooltip State=<'on' | 'off'>")
		scmd:AddParam("tooltip","state",REQUIRED,STRICT,{"string"},{"on","On","Off","off"},"on")
		
	scmd:AddCommand("raise", FB_Command_Raise, "misc",
		"/FlexBar Raise Event=<event> Source=<source> In=<seconds>")
		-- populate the parameter list for this command
		-- arguments are: command, parameter name, required, strict, types, allowed, default
		scmd:AddParam("raise","event" ,REQUIRED,NON_STRICT,{"string"},NO_BOUNDS,NO_DEFAULT)
		scmd:AddParam("raise","source",OPTIONAL,NON_STRICT,NO_TYPES,NO_BOUNDS,NO_DEFAULT)
		scmd:AddParam("raise","in"    ,OPTIONAL,STRICT,{"number"},{lowerbound=0},0)
		
	scmd:AddCommand("disable", FB_Command_Disable, "misc",
		"/FlexBar Disable Button=<buttons> state=<'on' | 'off'>")
		-- populate the parameter list for this command
		-- arguments are: command, parameter name, required, strict, types, allowed, default
		scmd:AddParam("disable","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, NO_DEFAULT)
		scmd:AddParam("disable","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, NO_DEFAULT)
		scmd:AddParam("disable","state" ,REQUIRED,NON_STRICT,{"string"},{"on","On","off","Off"}, NO_DEFAULT)
		scmd:AddParam("disable","toggle",OPTIONAL,STRICT,{"string"},{"true"},NO_DEFAULT)

	scmd:AddCommand("hide", FB_Command_Hide, "appearance",
		"/FlexBar Hide Button=<buttons>")
		-- arguments are: command, parameter name, required, strict, types, allowed, default
		scmd:AddParam("hide","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("hide","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("hide","toggle",OPTIONAL,STRICT,{"string"},{"true"},nil)
		scmd:AddParam("hide","on"    ,OPTIONAL,NON_STRICT,{"string"},FBEventList,nil)
		scmd:AddParam("hide","target",OPTIONAL,NON_STRICT,nil,nil,nil)
		scmd:AddParam("hide","if"    ,OPTIONAL,NON_STRICT,{"string"},FBConditionList,nil)
		scmd:AddParam("hide","iftarg",OPTIONAL,NON_STRICT,nil,nil,nil)
		
	scmd:AddCommand("show", FB_Command_Show, "appearance",
		"/FlexBar Show Button=<buttons>")
		scmd:AddParam("show","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("show","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("show","toggle",OPTIONAL,STRICT,{"string"},{"true"},nil)
		scmd:AddParam("show","on"    ,OPTIONAL,NON_STRICT,{"string"},FBEventList,nil)
		scmd:AddParam("show","target",OPTIONAL,NON_STRICT,nil,nil,nil)
		scmd:AddParam("show","if"    ,OPTIONAL,NON_STRICT,{"string"},FBConditionList,nil)
		scmd:AddParam("show","iftarg",OPTIONAL,NON_STRICT,nil,nil,nil)
		
	scmd:AddCommand("fade", FB_Command_Fade, "appearance",
		"/FlexBar Fade Button=<buttons> Alpha=<alpha>")
		scmd:AddParam("fade","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("fade","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("fade","alpha" ,REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=10},nil)
		scmd:AddParam("fade","toggle",OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=10},nil)
		scmd:AddParam("fade","on"    ,OPTIONAL,NON_STRICT,{"string"},FBEventList,nil)
		scmd:AddParam("fade","target",OPTIONAL,NON_STRICT,nil,nil,nil)
		scmd:AddParam("fade","if"    ,OPTIONAL,NON_STRICT,{"string"},FBConditionList,nil)
		scmd:AddParam("fade","iftarg",OPTIONAL,NON_STRICT,nil,nil,nil)
		
	scmd:AddCommand("scale", FB_Command_Scale, "appearance",
		"/FlexBar Scale Button=<buttons> Scale=<scale>")
		scmd:AddParam("scale","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("scale","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("scale","scale" ,REQUIRED,STRICT,{"number"},{lowerbound=5,upperbound=50},nil)
		scmd:AddParam("scale","toggle",OPTIONAL,STRICT,{"number"},{lowerbound=5,upperbound=50},nil)
		scmd:AddParam("scale","reset" ,OPTIONAL,NON_STRICT,{"string"},{"true"},nil)
		scmd:AddParam("scale","on"    ,OPTIONAL,NON_STRICT,{"string"},FBEventList,nil)
		scmd:AddParam("scale","target",OPTIONAL,NON_STRICT,nil,nil,nil)
		scmd:AddParam("scale","if"    ,OPTIONAL,NON_STRICT,{"string"},FBConditionList,nil)
		scmd:AddParam("scale","iftarg",OPTIONAL,NON_STRICT,nil,nil,nil)
		
	scmd:AddCommand("shade", FB_Command_Shade, "appearance",
		"/FlexBar Shade Button=<buttons> Color=<[r g b]>")
		scmd:AddParam("shade","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("shade","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("shade","color" ,REQUIRED,STRICT,{"number","table"},{lowerbound=0,upperbound=10},nil)
		scmd:AddParam("shade","toggle",OPTIONAL,STRICT,{"number","table"},{lowerbound=0,upperbound=10},nil)
		scmd:AddParam("shade","reset" ,OPTIONAL,NON_STRICT,{"string"},{"true"},nil)
		scmd:AddParam("shade","on"    ,OPTIONAL,NON_STRICT,{"string"},FBEventList,nil)
		scmd:AddParam("shade","target",OPTIONAL,NON_STRICT,nil,nil,nil)
		scmd:AddParam("shade","if"    ,OPTIONAL,NON_STRICT,{"string"},FBConditionList,nil)
		scmd:AddParam("shade","iftarg",OPTIONAL,NON_STRICT,nil,nil,nil)
		
	scmd:AddCommand("lock", FB_Command_Lock, "movement",
		"/FlexBar Lock Button=<buttons>")
		scmd:AddParam("lock","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons},nil)
		scmd:AddParam("lock","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons},nil)
	scmd:AddCommand("unlock", FB_Command_Unlock, "movement",
		"/FlexBar Unlock Button=<buttons>")
		scmd:AddParam("unlock","button",OPTIONAL,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons},nil)
		scmd:AddParam("unlock","group" ,OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons},nil)
	scmd:AddCommand("moveabs", FB_Command_MoveABS, "movement",
		"/FlexBar MoveABS Button=<button> XX=<x-coord> YY=<y-coord>")
		scmd:AddParam("moveabs","button",REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("moveabs","xx"    ,REQUIRED,STRICT,{"number"},nil,nil)
		scmd:AddParam("moveabs","yy"    ,REQUIRED,STRICT,{"number"},nil,nil)
		scmd:AddParam("moveabs","on"    ,OPTIONAL,NON_STRICT,{"string"},FBEventList,nil)
		scmd:AddParam("moveabs","target",OPTIONAL,NON_STRICT,nil,nil,nil)
		scmd:AddParam("moveabs","if"    ,OPTIONAL,NON_STRICT,{"string"},FBConditionList,nil)
		scmd:AddParam("moveabs","iftarg",OPTIONAL,NON_STRICT,nil,nil,nil)
		
	scmd:AddCommand("moverel", FB_Command_MoveRel, "movement",
		"/FlexBar MoveRel Button=<button> trgBtn=<button> dX=<x-coord> dY=<y-coord>")
		scmd:AddParam("moverel","button",REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("moverel","trgbtn",REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("moverel","dx"    ,REQUIRED,STRICT,{"number"},nil,nil)
		scmd:AddParam("moverel","dy"    ,REQUIRED,STRICT,{"number"},nil,nil)
		scmd:AddParam("moverel","on"    ,OPTIONAL,NON_STRICT,{"string"},FBEventList,nil)
		scmd:AddParam("moverel","target",OPTIONAL,NON_STRICT,nil,nil,nil)
		scmd:AddParam("moverel","if"    ,OPTIONAL,NON_STRICT,{"string"},FBConditionList,nil)
		scmd:AddParam("moverel","iftarg",OPTIONAL,NON_STRICT,nil,nil,nil)
		
	scmd:AddCommand("movetomouse", FB_Command_MoveToMouse, "movement",
		"/FlexBar MoveRel Button=<button> target=<button> dX=<x-coord> dY=<y-coord>")
		scmd:AddParam("movetomouse","button",REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("movetomouse","dx"    ,REQUIRED,STRICT,{"number"},nil,nil)
		scmd:AddParam("movetomouse","dy"    ,REQUIRED,STRICT,{"number"},nil,nil)
		scmd:AddParam("movetomouse","on"    ,OPTIONAL,NON_STRICT,{"string"},FBEventList,nil)
		scmd:AddParam("movetomouse","target",OPTIONAL,NON_STRICT,nil,nil,nil)
		scmd:AddParam("movetomouse","if"    ,OPTIONAL,NON_STRICT,{"string"},FBConditionList,nil)
		scmd:AddParam("movetomouse","iftarg",OPTIONAL,NON_STRICT,nil,nil,nil)
		
	scmd:AddCommand("group", FB_Command_Group, "group",
		"/FlexBar Group Button=<buttons> Anchor=<button>")
		scmd:AddParam("group","button",REQUIRED,STRICT,{"number","table"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("group","anchor",REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons},nil)
		
	scmd:AddCommand("ungroup", FB_Command_Ungroup, "group",
		"/FlexBar Ungroup Group=<group>")
		scmd:AddParam("ungroup","group",REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		
	scmd:AddCommand("circlegroup", FB_Command_CircleGroup, "group",
		"/FlexBar CircleGroup Group=<group>")
		scmd:AddParam("circlegroup","group"  ,REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("circlegroup","padding",OPTIONAL,STRICT,{"number"},{lowerbound=0},0)
		
	scmd:AddCommand("horizontalgroup", FB_Command_HorizontalGroup, "group",
		"/FlexBar HorizontalGroup Group=<group> Height=<height> Padding=<pixels>")
		scmd:AddParam("horizontalgroup","group"  ,REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("horizontalgroup","padding",OPTIONAL,STRICT,{"number"},{lowerbound=0},0)
		scmd:AddParam("horizontalgroup","height" ,OPTIONAL,STRICT,{"number"},{lowerbound=1},1)
		
	scmd:AddCommand("verticalgroup", FB_Command_VerticalGroup, "group",
		"/FlexBar VerticalGroup Group=<group> Width=<width> Padding=<pixels>")
		scmd:AddParam("verticalgroup","group"  ,REQUIRED,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons}, nil)
		scmd:AddParam("verticalgroup","padding",OPTIONAL,STRICT,{"number"},{lowerbound=0},0)
		scmd:AddParam("verticalgroup","width" ,OPTIONAL,STRICT,{"number"},{lowerbound=1},1)
		
	scmd:AddCommand("listgroups", FB_Command_ListGroups, "group",
		"/FlexBar ListGroups [Group=<groups>]")
		scmd:AddParam("listgroups","group",OPTIONAL,STRICT,{"number"},{lowerbound=1,upperbound=FBNumButtons},nil)
		
	scmd:AddCommand("listevents", FB_Command_ListEvents, "events",
		"/FlexBar ListEvents")
		
	scmd:AddCommand("deleteevent", FB_Command_DeleteEvent, "events",
		"/FlexBar DeleteEvent event=<event number>")
		scmd:AddParam("deleteevent","event",REQUIRED,STRICT,{"number"},{lowerbound=1},nil)
		
	scmd:AddCommand("mischelp",  FB_Command_DisplayMisc, "main",
		"/FlexBar MiscHelp - for miscellaneous options")
	scmd:AddCommand("eventhelp", FB_Command_DisplayEvents, "main",
		"/FlexBar EventHelp - for options on events")
	scmd:AddCommand("grouphelp", FB_Command_DisplayGroup, "main",
		"/FlexBar GroupHelp - for options on grouping buttons")
	scmd:AddCommand("appearance", FB_Command_DisplayAppearance, "main",
		"/FlexBar Appearance - for options on button appearance")
	scmd:AddCommand("movement", FB_Command_DisplayMovement, "main",
		"/FlexBar Movement - for options on moving buttons")

	scmd:AddCommand("test", FB_Command_Test, "test",
		"/Flexbar Test arg1=<string> arg2=<number> arg3=<optional> arg4=<required>")
		scmd:AddParam("test","arg1",true,nil,{"string"},nil, nil)
		scmd:AddParam("test","arg2",true,nil,{"number"},{lowerbound=5, upperbound=10}, nil)
		scmd:AddParam("test","arg3",false,nil,{"number", "table"},{1,3,5}, 10)
		scmd:AddParam("test","arg4",true,true,{"string", "table"},{"true", "false"}, nil)
end

function FB_Command_Test(msg)
	local args = FBcmd:GetParameters(msg)
	if not FBcmd:CheckParams("test", args) then
		util:Print("not valid")
	else
		util:Print("valid")
	end
end

-- Usage display functions
function FB_Command_DisplayMisc()
	FBcmd:DisplayUsage("misc")
	util:Print("For Show/Hide Grid, lock and text, group=<group> may be substituted for Button=<...>")
end

function FB_Command_DisplayAppearance()
	FBcmd:DisplayUsage("appearance")
	util:Print("For all commands, group=<group> may be substituted for Button=<...>")
end

function FB_Command_DisplayGroup()
	FBcmd:DisplayUsage("group")
	util:Print("The anchor button MUST be a part of the proposed group")
end

function FB_Command_DisplayMovement()
	FBcmd:DisplayUsage("movement")
	util:Print("Once grouped, only the anchor may be locked/unlocked")
	util:Print("MoveXXX only takes positive arguments currently")
end

function FB_Command_DisplayEvents()
	FBcmd:DisplayUsage("events")
end

-- Appearance command handlers

function FB_Command_Hide(msg)
-- Hide the buttons specified in Button=<buttons>
	local args = FBcmd:GetParameters(msg)
-- check basic usage
	if not FBcmd:CheckParameters("hide",args) then return end
-- In order to allow group= and button= interchangeably they had to both be optional
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
-- If the user specified On= ..... event will be populated with the dispatch info and function returns
	if FB_GetEventMsg("hide", args) then return end
-- get button list
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end
	
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		-- Toggle code - if hidden and toggle exists then unhide
		if args["toggle"] and button.FB.State["hidden"] then
			button.FB.State["hidden"] = nil
		else
			button.FB.State["hidden"] = true
		end
		-- Originally called ApplySettings at the end of the function - but the fact that
		-- On events go through here too requires that it not do potentially expensive operations
		-- So the call to apply the specific setting for only the buttons that are affected goes here
		FB_ApplyHidden(button, frame)
		-- And we only save the changed setting
		FBSavedProfile[UnitName("player")][buttons[index]].State["hidden"] = button.FB.State["hidden"]
		FB_Report(button:GetName() .. " hidden")
	end
end

function FB_Command_Show(msg)
-- Show the buttons specified in Button=<buttons>
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("show",args) then return end
-- In order to allow group= and button= interchangeably they had to both be optional
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
-- If the user specified On= ..... event will be populated with the dispatch info
	if FB_GetEventMsg("show", args) then return end
-- get button list
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end

	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		-- Toggle code - if not hidden and toggle exists then hide
		if args["toggle"] and not button.FB.State["hidden"] then
			button.FB.State["hidden"] = true
		else
			button.FB.State["hidden"] = nil
		end
		-- Originally called ApplySettings at the end of the function - but the fact that
		-- On events go through here too requires that it not do potentially expensive operations
		-- So the call to apply the specific setting for only the buttons that are affected goes here
		FB_ApplyHidden(button, frame)
		-- And we only save the changed setting
		FBSavedProfile[UnitName("player")][buttons[index]].State["hidden"] = button.FB.State["hidden"]
		FB_Report(button:GetName() .. " unhidden")
	end
end

function FB_Command_Fade(msg)
-- Change the alpah for the buttons specified in Button=<buttons>
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("fade",args) then return end
-- In order to allow group= and button= interchangeably they had to both be optional
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
-- If the user specified On= ..... event will be populated with the dispatch info
	if FB_GetEventMsg("fade", args) then return end

	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		-- Toggle code - if toggle, and button already has the set alpha, then toggle to toggle alpha
		if button.FB.State["alpha"] == nil then button.FB.State["alpha"] = 1 end
		if args["toggle"] and (button.FB.State["alpha"]*10) == args["alpha"] then
			button.FB.State["alpha"] = args["toggle"] / 10
		else
			button.FB.State["alpha"] = args["alpha"] / 10
		end
		-- Originally called ApplySettings at the end of the function - but the fact that
		-- On events go through here too requires that it not do potentially expensive operations
		-- So the call to apply the specific setting for only the buttons that are affected goes here
		FB_ApplyAlpha(button, frame)
		-- And we only save the changed setting
		FBSavedProfile[UnitName("player")][buttons[index]].State["alpha"] = button.FB.State["alpha"]
		FB_Report(button:GetName() .. format(" alpha set to %.2f",args["alpha"]/10))
	end
end

function FB_Command_Scale(msg)
-- Show the buttons specified in Button=<buttons>
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("scale",args) then return end
-- In order to allow group= and button= interchangeably they had to both be optional
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
-- If the user specified On= ..... event will be populated with the dispatch info
	if FB_GetEventMsg("scale", args) then return end

	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end
	
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		-- Toggle code - if toggle, and button already has the set alpha, then toggle to toggle alpha
		if button.FB.State["scale"] == nil then button.FB.State["scale"] = 1 end
		if args["toggle"] and (button.FB.State["scale"]*10) == args["scale"] then
			button.FB.State["scale"] = args["toggle"] / 10
		else
			button.FB.State["scale"] = args["scale"] / 10
		end
		if args["reset"] then
			button.FB.State["scale"] = nil
		end
		-- Originally called ApplySettings at the end of the function - but the fact that
		-- On events go through here too requires that it not do potentially expensive operations
		-- So the call to apply the specific setting for only the buttons that are affected goes here
		FB_ApplyScale(button, frame)
		-- And we only save the changed setting
		FBSavedProfile[UnitName("player")][buttons[index]].State["scale"] = button.FB.State["scale"]
		if button.FB.State["scale"] then 
			FB_Report(button:GetName() .. format(" scale set to %.2f",args["scale"]/10)) 
		else
			FB_Report(button:GetName() .. format(" scale is reset"))
		end
	end
end

function FB_Command_Shade(msg)
-- Shade the buttons specified in Button=<buttons> with the color in Color=
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("shade",args) then return end
-- In order to allow group= and button= interchangeably they had to both be optional
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
-- advanced checks that aren't automatic
	if 	type(args["color"]) ~= "table" or args["color"][3] == nil or
		(args["toggle"] and (type(args["toggle"]) ~= "table" or args["toggle"][3] == nil)) then
		util:Print("Error: Color not in correct format")
		return
	end
-- If the user specified On= ..... event will be populated with the dispatch info
	if FB_GetEventMsg("shade", args) then return end
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end
	local index
	for index = 1,3 do
		args["color"][index] = args["color"][index] / 10
		if args["toggle"] then args["toggle"][index] = args["toggle"][index] / 10 end
	end
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		-- Toggle code - if toggle, and button already the color specified, then toggle then go to toggle color
		if button.FB.State["icon"] == nil then button.FB.State["icon"] = {1, 1, 1} end
		if 	args["toggle"] and 
			args["color"][1] == button.FB.State["icon"][1] and
			args["color"][2] == button.FB.State["icon"][2] and
			args["color"][3] == button.FB.State["icon"][3] then
			args["color"] = args["toggle"]
		elseif args["reset"] then
			args["color"] = nil
			FB_Report(button:GetName() .. " has had its color reset")
		end
		if args["color"] then 
			button.FB.State["icon"] = util:TableCopy(args["color"]) 
			FB_Report(button:GetName() .. format(" color set to %.2f, %.2f, %.2f",
				args["color"][1],args["color"][2],args["color"][3]))
		else
			button.FB.State["icon"] = nil
		end
		-- Simply having the state in effect will change the color, but save the changed setting
		-- and avoid a call to apply settings
		if button.FB.State["color"] then
			FBSavedProfile[UnitName("player")][buttons[index]].State["color"] = util:TableCopy(button.FB.State["color"])
		else
			FBSavedProfile[UnitName("player")][buttons[index]].State["color"] = button.FB.State["color"]
		end
	end
end

-- Movement command handlers

function FB_Command_Lock(msg)
-- Lock the buttons specified in Button=<buttons>
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("lock",args) then return end
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end

	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		button.FB.State["locked"] = true
		FB_Report(button:GetName() .. " locked")
	end
	
	FB_ApplySettings()
end

function FB_Command_Unlock(msg)
-- Unlock the buttons specified in Button=<buttons>
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("lock",args) then return end
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		button.FB.State["locked"] = nil
		FB_Report(button:GetName() .. " unlocked")
	end
	
	FB_ApplySettings()
end

function FB_Command_MoveABS(msg)
-- Move the specified button to a point relative to the bottom left of the screen
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("moveabs",args) then return end
-- If the user specified On= ..... event will be populated with the dispatch info and function returns
	if FB_GetEventMsg("moveabs", args) then return end

	local button, frame = FB_GetWidgets(args["button"])
	-- if the button is in a group, but not the anchor, return
	if button.FB.State["group"] and button.FB.State["group"] ~= args["button"] then return end
	FB_MoveButtonABS(button, args["xx"], args["yy"])
	FB_Report(button:GetName() .. format(" moved to x=%d, y=%d", args["xx"], args["yy"]))
	FB_ResyncSettings()
	if button.FB.State["group"] then 
		FBGroupData[button.FB.State["group"]] = FB_GetBoundingButtons(button.FB.State["group"]) 
		FB_CheckGroups()
	end
end

function FB_Command_MoveRel(msg)
-- Move the specified button to the location relative to target
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("moverel",args) then return end
-- If the user specified On= ..... event will be populated with the dispatch info and function returns
	if FB_GetEventMsg("moverel", args) then return end

	local button, frame = FB_GetWidgets(args["button"])
	-- if the button is in a group, but not the anchor, return
	if button.FB.State["group"] and button.FB.State["group"] ~= args["button"] then return end
	local target, frame = FB_GetWidgets(args["trgbtn"])
	args["dx"] = args["dx"] / UIParent:GetScale()
	args["dy"] = args["dy"] / UIParent:GetScale()
	FB_MoveButtonRel(button, target, args["dx"], args["dy"])
	FB_Report(button:GetName() .. format(" moved to x=%d, y=%d", args["dx"], args["dy"]) .. " with respect to " .. target:GetName())
	FB_ResyncSettings()
	if button.FB.State["group"] then 
		FBGroupData[button.FB.State["group"]] = FB_GetBoundingButtons(button.FB.State["group"]) 
		FB_CheckGroups()
	end
end

function FB_Command_MoveToMouse(msg)
-- Move the specified button to the location relative to mouse
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("movetomouse",args) then return end
-- If the user specified On= ..... event will be populated with the dispatch info and function returns
	if FB_GetEventMsg("movetomouse", args) then return end

	local button, frame = FB_GetWidgets(args["button"])
	-- if the button is in a group, but not the anchor, return
	if button.FB.State["group"] and button.FB.State["group"] ~= args["button"] then return end
	local x,y = GetCursorPosition(UIParent)
	x = x / UIParent:GetScale()
	y = y / UIParent:GetScale()
	args["dx"] = args["dx"] / UIParent:GetScale()
	args["dy"] = args["dy"] / UIParent:GetScale()
	FB_MoveButtonABS(button, x+args["dx"], y+args["dy"])
	FB_Report(button:GetName() .. format(" moved to x=%d, y=%d", x+args["dx"], y+args["dy"]))
	FB_ResyncSettings()
	if button.FB.State["group"] then 
		FBGroupData[button.FB.State["group"]] = FB_GetBoundingButtons(button.FB.State["group"]) 
		FB_CheckGroups()
	end
end

-- All group related slash command handlers here

function FB_Command_CircleGroup(msg)
-- Arrange the specified group into a circular formation
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("circlegroup",args) then return end
	local button, frame = FB_GetWidgets(args["group"])
	if button.FB.State["group"] ~= FB_GetButtonNum(button) then
		util:Echo("No such group", "green")
		return
	end
	FB_CircularGroup(args["group"], args["padding"])
	FB_Report(args["group"] .. " auto-arranged")
	FB_ApplySettings()
	FBGroupData[args["group"]] = FB_GetBoundingButtons(args["group"])
end

function FB_Command_VerticalGroup(msg)
-- Arrange the specified group into a vertical formation
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("verticalgroup",args) then return end
	local button, frame = FB_GetWidgets(args["group"])
	if button.FB.State["group"] ~= FB_GetButtonNum(button) then
		util:Echo("No such group", "green")
		return
	end
	FB_VerticalGroup(args["group"], args["width"], args["padding"])
	FB_Report(args["group"] .. " auto-arranged")
	FB_ResyncSettings()
	FBGroupData[args["group"]] = FB_GetBoundingButtons(args["group"])
end

function FB_Command_HorizontalGroup(msg)
-- Arrange the specified group into a horizontal formation
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("horizontalgroup",args) then return end
	local button, frame = FB_GetWidgets(args["group"])
	if button.FB.State["group"] ~= FB_GetButtonNum(button) then
		util:Echo("No such group", "green")
		return
	end
	FB_HorizontalGroup(args["group"], args["height"], args["padding"])
	FB_Report(args["group"] .. " auto-arranged")
	FB_ResyncSettings()
	FBGroupData[args["group"]] = FB_GetBoundingButtons(args["group"])
end

function FB_Command_Group(msg)
-- Set a group for the buttons specified in Button=<buttons>
-- Group is anchored on button specified in Anchor=<button>
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("group",args) then return end
	local buttons = FB_GetButtonList(args["button"])
-- extended checks
-- Groups must have more than one members
	if buttons.n < 2 then 
		util:Echo("Error: Groups must have more than one member")
		return
	end
-- Anchor must also be a member of the set of buttons being made into a group
	local index
	local goodanchor = false
	for index = 1,buttons.n do
		if args["anchor"] == buttons[index] then goodanchor = true end
	end
	
	if not goodanchor then
		util:Echo("Error: Anchor must be a member of the buttons being formed into a group")
		return
	end
	
-- Good parameters - set the group
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		button.FB.State["group"] = args["anchor"]
	end
	
-- Reform groups
	FB_ReformGroups()
	if FBVerbose then FB_Command_ListGroups("group="..args["anchor"]) end
end

function FB_Command_Ungroup(msg)
-- Ungroup the buttons specified in Button=<buttons>, and their groupmates
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("ungroup",args) then return end
	local buttons = FB_GetButtonList(args["group"])
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		if button.FB.State["group"] == buttons[index] then
			FB_Report("Disbanding group " .. buttons[index])
			local members = FB_GetGroupMembers(button.FB.State["group"])
			local index2
			for index2 = 1, members.n do
				local button, frame = FB_GetWidgets(members[index2])
				button.FB.State["group"] = nil
				button.FB.State["locked"] = nil
			end
		end
	end
	FB_ReformGroups()
end

function FB_Command_ListGroups(msg)
-- Show buttons in the groups in Group=
-- show all groups if group= ommitted
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("listgroups",args) then return end

	if (args["group"] == nil) then
		local index, value
		local groups = {}
		for index = 1, FBNumButtons do
			local button, frame = FB_GetWidgets(index)
			if button.FB.State["group"] == index then groups[index] = true end
		end
		local text = "The following groups are present:"
		for index = 1, FBNumButtons do
			if groups[index] then text = text .. " " .. index end
		end
		util:Print(text)
		return
	end

	if type(args["group"]) == "number" then
		buttons= FB_GetGroupMembers(args["group"])
	else
		FBcmd:DisplayUsage("group")
		return
	end
	
	local index
	local text = "Buttons in group " .. args["group"] .. " :"
	for index = 1, buttons.n do
		text = text .. " " ..  buttons[index]
	end
	util:Print(text)
end

-- Event functions

function FB_Command_ListEvents(msg)
-- Show events
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("listevents",args) then return end
	
	local index, event
	for index, event in ipairs(FBEvents) do
		util:Print(index .. ") On " .. event["on"] .. " " .. event["targettext"] .. " : " .. event["command"])
	end
end

function FB_Command_DeleteEvent(msg)
-- Delete specified event from Button in Button=
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("deleteevent",args) then return end
	table.remove(FBEvents,args["event"])
	FB_Report("Event " .. args["event"] .. " deleted")
	FBSavedProfile[UnitName("player")].Events = util:TableCopy(FBEvents)
end

-- Miscellaneous command handlers

function FB_Command_Verbose(msg)
-- Set the feedback verbosity level
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("verbose",args) then return end
	if args["state"] ~= nil then
		if string.lower(args["state"]) == "on" then
			FBVerbose = true
			util:Print("Verbose mode Active", "green")
		else
			FBVerbose = nil;
			util:Print("Verbose mode Inactive", "yellow")
		end
	else
		FBcmd:DisplayUsage("misc")
	end
end

function FB_Command_Tooltip(msg)
-- Set the feedback verbosity level
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("verbose",args) then return end
	if args["state"] ~= nil then
		if string.lower(args["state"]) == "on" then
			FBNoTooltips = nil
			util:Print("Tooltips will now show", "green")
		else
			FBNoTooltips = true
			util:Print("Tooltips will not show", "yellow")
		end
	else
		FBcmd:DisplayUsage("misc")
	end
end

function FB_Command_LockIcon(msg)
-- Lock the icon to the buttons specified in Button=<buttons>
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("lockicon",args) then return end
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end
	
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		if args["off"] then
			button.FB.State["lockedicon"] = nil
			FB_Report(button:GetName() .. " has it's Icon unlocked click and drag to remove")
		else
			button.FB.State["lockedicon"] = true
			FB_Report(button:GetName() .. " has it's Icon locked - shift click to remove")
		end
	end
	
	FB_ApplySettings()
end

function FB_Command_Text(msg)
-- Change hotkey text to the specified text
-- %d is replaced by the button number, %s is replaced by the binding
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("text",args) then return end
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
	if (args["text"]==nil) then
		args["text"] = ""
	end
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		button.FB.State["hotkeytext"] = args["text"]
		FB_Report(button:GetName() .. " now has " .. args["text"] .. " as it's hotkey")
	end
	
	FB_ApplySettings()
end

function FB_Command_ReMap(msg)
-- Remap the bonus buttons to the specified base
	local args = FBcmd:GetParameters(msg)
-- auto check parameters
	if not FBcmd:CheckParameters("remap", args) then return end
-- in order to have button= and group= interchangeable, we must make them optional and
-- explicilty check them here.
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end

-- Get button list
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"]) then
		buttons= FB_GetGroupMembers(args["group"])
	end
-- Another check that can't be automated
	if args["base"] + buttons.n - 1 > 120 then
		util:Print("Error: Remap would make a button ID out of bounds")
		return
	end
-- If the user specified On= ..... event will be populated with the dispatch info
	if FB_GetEventMsg("remap", args) then return end
	local index
	for index = 1, buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		-- if toggle is present AND button are already mapped to anything but default OR Reset is true
		-- then map them to the default, otherwise map them to base + index - 1.
		if 	(args["toggle"] and button.FB.State["remap"]) or args["reset"] then
			button.FB.State["remap"] = nil
			button:SetID(button.FB.State["oldid"])
		else
			button.FB.State["remap"] = index + args["base"] - 1
			button:SetID(button.FB.State["remap"])
		end
		local saved = this
		this = button
		FlexBarButton_Update()
		this = button
		FlexBarButton_UpdateState()
		this = saved
		FBSavedProfile[UnitName("player")][buttons[index]].State["remap"] = button.FB.State["remap"]
		FB_Report(button:GetName() .. " remapped to " .. button:GetID())
	end
end

function FB_Command_HideGrid(msg)
-- Hide the button background if it doesn't have an action
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("hidegrid",args) then return end
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end
	
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		if args["toggle"] and button.FB.State["hidegrid"] then
			button.FB.State["hidegrid"] = nil
		else
			button.FB.State["hidegrid"] = true
		end
		if button.FB.State["hidegrid"] then
			button.showgrid=0
		else
			button.showgrid=1
			button:Show()
		end
		local saved = this
		this = button
		FlexBarButton_Update()
		this = button
		FlexBarButton_UpdateState()
		this = saved
		FB_Report(button:GetName() .. " has it's background hidden.  It will show when you drag an item")
	end
	
	FB_ApplySettings()
end

function FB_Command_ShowGrid(msg)
-- Hide the button background if it doesn't have an action
	local args = FBcmd:GetParameters(msg)
-- Check basic usage
	if not FBcmd:CheckParameters("showgrid",args) then return end
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end
	
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		if args["toggle"] and not button.FB.State["hidegrid"] then
			button.FB.State["hidegrid"] = true
		else
			button.FB.State["hidegrid"] = nil
		end
		if button.FB.State["hidegrid"] then
			button.showgrid=0
		else
			button.showgrid=1
			button:Show()
		end
		local saved = this
		this = button
		FlexBarButton_Update()
		this = button
		FlexBarButton_UpdateState()
		this = saved
		FB_Report(button:GetName() .. " has it's background hidden.  It will show when you drag an item")
	end
	
	FB_ApplySettings()
end

function FB_Command_Advanced(msg)
-- Set it so that only a left click on this button will trigger it
	local args = FBcmd:GetParameters(msg)
	if not FBcmd:CheckParameters("advanced", args) then return end
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end

	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		if string.lower(args["state"]) == "on" then
			button.FB.State["advanced"] = true
			FB_Report(button:GetName() .. " will now only fire on a left click")
		else
			button.FB.State["advanced"] = nil;
			FB_Report(button:GetName() .. " will now treat all clicks the same")
		end
	end
	FB_ApplySettings()
end

function FB_Command_Use(msg)
-- Use the buttons specified in Button=<buttons>
-- NOTE - this appears to ONLY work on
	local args = FBcmd:GetParameters(msg)
	if not FBcmd:CheckParameters("use", args) then return end
-- check here for one or the other
	if (args["group"]==nil) and (args["button"]==nil) then
		util:Print("Error: You must specify a group or button")
	end
-- If the user specified On= ..... event will be populated with the dispatch info
	if FB_GetEventMsg("use", args) then return end
-- get button list
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		local id = button:GetID()
		MacroFrame_EditMacro()
		UseAction(id, 1)
	end
end

function FB_Command_ResetAll(msg)
-- Reset entire profile for character
	local args = FBcmd:GetParameters(msg)
	if not FBcmd:CheckParameters("resetall", args) then return end

	FBSavedProfile[UnitName("player")] = nil
	ReloadUI()
end

function FB_Command_LoadConfig(msg)
-- Load a named config from the config file
-- the naive way to do this is a simple for loop - however that cause a small
-- problem by executing group commands to fast.
-- altered to do one command per second.
	local args = FBcmd:GetParameters(msg)
	if not FBcmd:CheckParameters("loadconfig", args) then return end
-- un auto-matable checks
	if type(getglobal(args["config"])) ~= "table" then
		util:Print("Error: " .. args["config"] .. " not a valid config file")
	end
	
	if nextcommand ~= 0 then
		util:Echo("Error: Still processing last config file")
		return
	end

	currentconfig 	= getglobal(args["config"])
	local index, value
	local count = 0
	for index, value in ipairs(currentconfig) do
		count=count+1
	end
	currentconfig.n = count
	FBTimers["loadconfig"] = Timer_Class:New(1, nil, nil, nil, FB_Config) 
end

function FB_Config()
	nextcommand = nextcommand + 1
	if nextcommand > currentconfig.n then
		nextcommand = 0
		currentconfig = nil
		return
	end
	
	util:Echo(currentconfig[nextcommand], "green")
	FBcmd:Dispatch(currentconfig[nextcommand])
	FBTimers["loadconfig"] = Timer_Class:New(1, nil, nil, nil, FB_Config) 
end

function FB_Command_Disable(msg)
-- Disable the regular action on a key
	local args = FBcmd:GetParameters(msg)
	if not args["skipboundschecking"] then
		-- basic usage test
		if not FBcmd:CheckParameters("disable", args) then return end 
		-- Place advanced bounds checking here
		-- check here for one or the other
		if (args["group"]==nil) and (args["button"]==nil) then
			util:Print("Error: You must specify a group or button")
		end
	end
	local buttons = FB_GetButtonList(args["button"])
	if (args["group"] and (type(args["group"]) == "number")) then
		buttons= FB_GetGroupMembers(args["group"])
	end

	-- check for event here - return if specified.  FBEvents will be updated
	if FB_GetEventMsg("disable", args) then return end
	-- check for conditional here
--	if not FB_CheckConditional(args) then return end
	local index
	for index = 1,buttons.n do
		local button, frame = FB_GetWidgets(buttons[index])
		-- Toggle code - if disabled and toggle exists then enable
		if (args["toggle"] and button.FB.State["disabled"]) or string.lower(args["state"]) == 'off' then
			button.FB.State["disabled"] = nil
			FB_Report(button:GetName() .. " enabled")
		elseif (args["toggle"] and not button.FB.State["disabled"]) or string.lower(args["state"]) == 'on' then
			button.FB.State["disabled"] = true
			FB_Report(button:GetName() .. " disabled")
		end
		-- And we only save the changed setting
		FBSavedProfile[UnitName("player")][buttons[index]].State["disabled"] = button.FB.State["disabled"]
	end
end 

function FB_Command_Raise(msg)
-- Raise an event
	local args = FBcmd:GetParameters(msg)
	-- event commands are going to have this set to true
	if not args["skipboundschecking"] then
		-- basic usage test
		if not FBcmd:CheckParameters("template", args) then return end 
		-- Place advanced bounds checking here
	end
	-- check for event here - return if specified.  FBEvents will be updated
	if FB_GetEventMsg("template", args) then return end
	-- check for conditional here
--	if not FB_CheckConditional(args) then return end
	if args["in"] then
		local name = args["event"] .. format("%.2f",GetTime())
		FBTimers[name] = Timer_Class:New(args["in"],false,nil,nil,function() FB_RaiseEvent(args["event"],args["source"]) end)
		FB_Report("Raising " .. args["event"] .. " in " .. args["in"] .. " seconds");
	else
		FB_RaiseEvent(args["event"],args["source"])
		FB_Report("Raised " .. args["event"])
	end
end 

function FB_Command_SafeLoad(msg)
-- Enable Safeload
	local args = FBcmd:GetParameters(msg)
	-- basic usage test
	if not FBcmd:CheckParameters("safeload", args) then return end 

	if string.lower(args["state"]) == "on" then
		FBSafeLoad = true
		util:Echo("SAFE LOAD ENGAGED -- you will need to manually load your profile now", "green")
	else
		FBSafeLoad = nil
		util:Echo("SAFE LOAD DIS-ENGAGED -- profile will autoload now, if you experience lost configurations, re-enable safe load", "green")
	end
end 

function FB_Command_LoadProfile(msg)
-- load a profile and make current
	local args = FBcmd:GetParameters(msg)
	-- basic usage test
	if not FBcmd:CheckParameters("loadprofile", args) then return end 
	
	if not FBSavedProfile[args["profile"]] then
		util:Echo("No such profile: " .. args["profile"], "yellow")
	else
		FBSavedProfile[UnitName("player")] = util:TableCopy(FBSavedProfile[args["profile"]])
		util:Echo("Making " .. args["profile"] .. " the current profile and reloading", "green")
		FB_LoadProfile()
	end
end 

function FB_Command_SaveProfile(msg)
-- Save the current profile
	local args = FBcmd:GetParameters(msg)
	-- basic usage test
	if not FBcmd:CheckParameters("saveprofile", args) then return end 
	
	FBSavedProfile[args["profile"]] = util:TableCopy(FBSavedProfile[UnitName("player")])
	util:Echo("Profile: " .. args["profile"] .. " saved" )
end 

function FB_Command_Template(msg)
-- template for new commands
	local args = FBcmd:GetParameters(msg)
	-- event commands are going to have this set to true
	if not args["skipboundschecking"] then
		-- basic usage test
		if not FBcmd:CheckParameters("template", args) then return end 
		-- Place advanced bounds checking here
	end
	-- check for event here - return if specified.  FBEvents will be updated
	if FB_GetEventMsg("template", args) then return end
	-- check for conditional here
--	if not FB_CheckConditional(args) then return end



	-- perform action here
end 
