--[[

    Wardrobe

    By Cragganmore
    Based on "Tackle Box" by Mugendai and "BCUI Tracking Menu" by Kugnar

    Wardrobe lets you define up to 10 distinct equipment profiles 
    (called "outfits") and lets you switch among them on the fly.  
    For example, you can define a Normal Outfit that consists of 
    your regular equipment, an Around Town Outfit that consists of 
    what you'd like to wear when inside a city or roleplaying, a 
    Stamina Outfit that consists of all your best +stam gear, etc.  
    You can then switch amongst these outfits using a simple slash chat 
    command (/wardrobe wear Around Town Outfit), or using a small 
    interactive button docked beneath your radar.

   ]]--

WARDROBE_VERSION = "1.1";
   
WARDROBE_DEBUG = false;
WARDROBE_NUM_OUTFITS = 10;
WARDROBE_NUM_POPUP_FUNCTION_BUTTONS = 5;

local InventorySlots = {"HeadSlot","NeckSlot","ShoulderSlot","BackSlot","ChestSlot","ShirtSlot",
                        "TabardSlot","WristSlot","HandsSlot","WaistSlot","LegsSlot","FeetSlot",
                        "Finger0Slot","Finger1Slot","Trinket0Slot","Trinket1Slot","MainHandSlot",
                        "SecondaryHandSlot","RangedSlot"};

-- the variable that stores all the wardrobe info
-- and gets saved when you quit the game
Wardrobe_Config = {};
Wardrobe_Config.Enabled = true;
Wardrobe_Config.xOffset = 10;
Wardrobe_Config.yOffset = 39;

Wardrobe_Current_Outfit = 0;
Wardrobe_AlreadySetCharactersWardrobeID = false;    -- set this to true once we've looked up this character's wardrobe info
Wardrobe_PopupFunction = "";
WARDROBE_POPUP_TITLE = "";
Wardrobe_Rename_OldName = "";
Wardrobe_BeingDragged = false;
Wardrobe_DragLock = false;
Wardrobe_InCombat = false;

--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              INITIALIZATION FUNCTIONS                                      --
--                                                                                            --
--============================================================================================--
--============================================================================================--


---------------------------------------------------------------------------------
--
-- Stuff done when the plugin first loads
-- 
---------------------------------------------------------------------------------
function Wardrobe_OnLoad()  

    -- watch our bags and update our wardrobe availability
    this:RegisterEvent("BAG_UPDATE");
    this:RegisterEvent("UNIT_INVENTORY_CHANGED");
    this:RegisterEvent("PLAYER_REGEN_DISABLED");
    this:RegisterEvent("PLAYER_LEAVE_COMBAT");
        
    Wardrobe_RegisterCosmos();
end


---------------------------------------------------------------------------------
--
-- Register with Cosmos UI
-- 
---------------------------------------------------------------------------------
function Wardrobe_RegisterCosmos()

    if ( Cosmos_RegisterConfiguration ) then 

        Cosmos_RegisterConfiguration(
            "COS_WARDROBE",
            "SECTION",
            WARDROBE_CONFIG_HEADER,
            WARDROBE_CONFIG_HEADER_INFO
        );
        Cosmos_RegisterConfiguration(
            "COS_WARDROBE_HEADER",
            "SEPARATOR",
            WARDROBE_CONFIG_HEADER,
            WARDROBE_CONFIG_HEADER_INFO
        );
        Cosmos_RegisterConfiguration(
            "COS_WARDROBE_ENABLED",
            "CHECKBOX",
            WARDROBE_CONFIG_ENABLED,
            WARDROBE_CONFIG_ENABLED_INFO,
            Wardrobe_Toggle,
            Wardrobe_Config.Enabled
        );

        local WardrobeCommands = {"/wardrobe","/wd"};
        Cosmos_RegisterChatCommand (
            "WARDROBE_COMMANDS", -- Some Unique Group ID
            WardrobeCommands, -- The Commands
            Wardrobe_ChatCommandHandler,
            WARDROBE_CHAT_COMMAND_INFO -- Description String
        );
        
    -- if cosmos isn't installed, create the slash commands
    else
        SlashCmdList["WARDROBESLASH"] = Wardrobe_ChatCommandHandler;
        SLASH_WARDROBESLASH1 = "/wardrobe";
        SLASH_WARDROBESLASH2 = "/wd";  
    end
    
    -- makes sure we have a Print() command, just in
    -- case cosmos isn't installed
    if(not Print) then
        function Print(msg, r, g, b, frame) 
            if (not r) then r = 1.0; end
            if (not g) then g = 1.0; end
            if (not b) then b = 1.0; end
            if ( frame ) then 
                frame:AddMessage(msg,r,g,b);
            else
                if ( DEFAULT_CHAT_FRAME ) then 
                    DEFAULT_CHAT_FRAME:AddMessage(msg, r, g, b);
                end
            end
        end
    end
end


---------------------------------------------------------------------------------
--
-- Event handler
-- 
---------------------------------------------------------------------------------
function Wardrobe_OnEvent(event)

    if (Wardrobe_Config.Enabled) then       
        if (event == "PLAYER_REGEN_DISABLED") then
            Wardrobe_InCombat = true;
        elseif (event == "PLAYER_LEAVE_COMBAT") then
            Wardrobe_InCombat = false;
            
        -- don't update outfit availability if in combat b/c this can cause slowdown with players using ammo (constantly updating their availability with every shot)
        elseif (not Wardrobe_InCombat and UnitName("player") ~= "Unknown Being" and UnitName("player") ~= "Unknown Entity" and (event == "BAG_UPDATE" or event == "UNIT_INVENTORY_CHANGED")) then
            --Wardrobe_UpdateOutfitAvailability();
        elseif (event == "VARIABLES_LOADED") then
            Wardrobe_ButtonSetLocation();      
        end
    end
end


-----------------------------------------------------------------------------------
--
-- Workaround functions to handle a problem where the money field of a tooltip
-- will get hidden whenever tooltips are invoked.
-- 
-- Code by Telo.  From his post (http://forums.worldofwarcraft.com/thread.aspx?fn=wow-interface-customization&t=16768&tmp=1#post16768):
--   Whenever a tooltip is set, the game sends a CLEAR_TOOLTIP event, which causes GameTooltip_OnEvent to call 
--   GameTooltip_ClearMoney, hiding the money frame for GameTooltip. There doesn't appear to be any identifying 
--   information sent with the CLEAR_TOOLTIP event that would allow GameTooltip_OnEvent to discriminate between 
--   its tooltip and others, so simply hooking this function doesn't appear to help. I've worked around the 
--   issue for my code by using something similar to the following code wherever I use a hidden tooltip.
--
-----------------------------------------------------------------------------------
local lOriginal_GameTooltip_ClearMoney;
local function Wardrobe_MoneyToggle()
    if( lOriginal_GameTooltip_ClearMoney ) then
        GameTooltip_ClearMoney = lOriginal_GameTooltip_ClearMoney;
        lOriginal_GameTooltip_ClearMoney = nil;
    else
        lOriginal_GameTooltip_ClearMoney = GameTooltip_ClearMoney;
        GameTooltip_ClearMoney = Wardrobe_GameTooltip_ClearMoney;
    end
end
function Wardrobe_GameTooltip_ClearMoney()
    -- Intentionally empty; don't clear money while we use hidden tooltips
end


---------------------------------------------------------------------------------
--
-- Update function for showing/hiding the popup menu
-- 
---------------------------------------------------------------------------------
function Wardrobe_OnUpdate()
    if (not MouseIsOver(Wardrobe_Popup) and not MouseIsOver(Wardrobe_IconFrame)) then
        Wardrobe_Popup:Hide();
    end
end


---------------------------------------------------------------------------------
--
-- Cleanup function to remove out of date data from previous AddOn version
-- 
---------------------------------------------------------------------------------
function wardrobe_FilterUnusedoutfits()

    if (not Wardrobe_AlreadyFiltered) then
        numOutfitSlots = table.getn(Wardrobe_Config[charID].Outfit);
        WardrobeDebug("Before processing: total outfitSlots = "..numOutfitSlots);
        for i = 1, numOutfitSlots do
            WardrobeDebug("Outfit in slot "..i.." = ["..Wardrobe_Config[charID].Outfit[i].OutfitName.."]");
        end    

        WardrobeDebug("Begin processing.");

        numOutfitSlots = table.getn(Wardrobe_Config[charID].Outfit);
        local currentSlot = 1;
        for i = 1, numOutfitSlots do

            -- if we found an empty outfit
            if (Wardrobe_Config[charID].Outfit[currentSlot].OutfitName == "") then

                WardrobeDebug("Found blank outfit at "..i);

                -- remove the outfit
                table.remove(Wardrobe_Config[charID].Outfit, currentSlot);
            else
                currentSlot = currentSlot + 1
            end
        end

        WardrobeDebug("Done processing.");
        numOutfitSlots = table.getn(Wardrobe_Config[charID].Outfit);
        WardrobeDebug("After processing: total outfitSlots = "..numOutfitSlots);
        for i = 1, numOutfitSlots do
            WardrobeDebug("Outfit in slot "..i.." = ["..Wardrobe_Config[charID].Outfit[i].OutfitName.."]");
        end    
        
        Wardrobe_AlreadyFiltered = true;
    end
end


--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              CHAT COMMAND FUNCTIONS                                        --
--                                                                                            --
--============================================================================================--
--============================================================================================--


---------------------------------------------------------------------------------
--
-- Break a chat command into its command and variable parts (i.e. "debug on" 
-- will break into command = "debug" and variable = "on", or "add my spiffy wardrobe"
-- breaks into command = "add" and variable = "my spiffy wardrobe"
-- 
---------------------------------------------------------------------------------
function Wardrobe_ParseCommand(msg)
    firstSpace = string.find(msg, " ", 1, true);
    if (firstSpace) then
        local command = string.sub(msg, 1, firstSpace - 1);
        local var  = string.sub(msg, firstSpace + 1);
        return command, var
    else
        return msg, nil;
    end
end


---------------------------------------------------------------------------------
--
-- A simple chat command handler.  takes commands in the form "/wardrobe command var"
-- 
---------------------------------------------------------------------------------
function Wardrobe_ChatCommandHandler(msg)
    local command, var = Wardrobe_ParseCommand(msg);
    if ((not command) and msg) then
        command = msg;
    end
    if (command) then
        command = string.lower(command);
        if (command == "add") then
            Wardrobe_AddNewOutfit(var);
        elseif (command == "delete") then
            Wardrobe_EraseOutfit(var);
        elseif (command == "reset") then
            Wardrobe_EraseAllOutfits();
        elseif (command == "list") then
            Wardrobe_ListOutfits(var);
        elseif (command == "switch" or command == "wear" or command == "use") then
            Wardrobe_WearOutfit(var);
        elseif (command == "update") then
            Wardrobe_UpdateOutfit(var);
        elseif (command == "avail") then
            Wardrobe_UpdateOutfitAvailability();
        elseif (command == "debug") then
            Wardrobe_ToggleDebug();
        elseif (command == "report") then
            Wardrobe_DumpDebugReport();
        elseif (command == "itemlist") then
            Wardrobe_BuildItemList();
        elseif (command == "on") then
            Wardrobe_Toggle(1);
        elseif (command == "off") then
            Wardrobe_Toggle(0);
        elseif (command == "lock") then
            Wardrobe_ToggleLockButton(true);
        elseif (command == "unlock") then
            Wardrobe_ToggleLockButton(false);
        elseif (command == "version") then
            Print("Wardrobe version "..WARDROBE_VERSION, 1.0, 1.0, 0.5);
        else
            Wardrobe_ShowHelp();
        end
    end
end


---------------------------------------------------------------------------------
--
-- Display the help text
-- 
---------------------------------------------------------------------------------
function Wardrobe_ShowHelp()
    Print("Wardrobe, an AddOn by Cragganmore, Version "..WARDROBE_VERSION, 1.0, 1.0, 0.5);
    Print("Wardrobe allows you to define and switch up to 10 different outfits.", 1.0, 1.0, 0.5);
    Print("Usage: /wardrobe <add/delete/wear/update/list/reset/lock/unlock>", 1.0, 1.0, 0.5);
    Print("   add [outfit name] - Define an outfit from what your are currently wearing.", 1.0, 1.0, 0.5);
    Print("   delete [outfit name] - Delete the specified outfit from your Wardrobe.", 1.0, 1.0, 0.5);
    Print("   wear [outfit name] - Wear the specified outfit.", 1.0, 1.0, 0.5);
    Print("   update [outfit name] - Update the specified outfit to reflect your currently equipped items.", 1.0, 1.0, 0.5);
    Print("   list - List your outfits.", 1.0, 1.0, 0.5);
    Print("   list items - List your outfits and all their items.", 1.0, 1.0, 0.5);
    Print("   reset - Delete all outfits in your wardrobe.", 1.0, 1.0, 0.5);
    Print("   lock/unlock - Lock or unlock moving the wardrobe button.", 1.0, 1.0, 0.5);
    Print("In the UI, outfit names are colored as follows:", 1.0, 1.0, 0.5);
    Print("   Bright Green: Your currently equipped outfit.", 1.0, 1.0, 0.5);
    Print("   Green: An outfit that has all its items available in your inventory.", 1.0, 1.0, 0.5);
    Print("   Grey: An outfit that is missing one or more items.", 1.0, 1.0, 0.5);
    Print("   Greyed outfits may still be equipped.  The missing items just won't be worn.", 1.0, 1.0, 0.5);
end


--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              WARDROBE TABLE FUNCTIONS                                      --
--                                                                                            --
--============================================================================================--
--============================================================================================--

-- NOTES:
--
-- For each character, the wardrobes are stored in a datastructure that looks like this
-- 
-- x = total number of outfits
-- y = total slots on a character (head, feet, hands, etc)
--
-- Outfit[x]        -- the datastructure for a single outfit
--
--     OutfitName   -- the name of this outfit
--     Available    -- true if all of the items in this outfit are in our bags or equiped
--     Item[1]      -- the data structure for all the items in this outfit
--        .
--        .
--        .
--     Item[y]      
--
-- So, let's say you have two outfits in your wardrobe.  Wardrobe[1] represents outfit 1, and Wardrobe[2] 
-- represents outfit 2.  for outfit 1, Wardrobe[1].OutfitName would be the name of this outfit (say, "In town outfit").  
-- the item on your character slot 5 would be Wardrobe[1].Item[5].


---------------------------------------------------------------------------------
--
-- Add an entry for this character to the main table of wardrobes
-- 
---------------------------------------------------------------------------------
function Wardrobe_AddThisCharacterToWardrobeTable()
    
    WardrobeDebug("Didn't find a wardrobe ID for this character.  Adding this character to the table...");
    
    -- build the structure for this char's wardrobe
    tempTable = { };
    tempTable.PlayerName = UnitName("player");
    tempTable.Outfit = { };
    
    -- stick this structure into the main table of wardrobes
    table.insert(Wardrobe_Config, tempTable);
end


---------------------------------------------------------------------------------
--
-- Create and return a blank outfit structure
-- 
---------------------------------------------------------------------------------
function Wardrobe_CreateBlankOutfit()
    local tempTable2 = { };
    tempTable2.OutfitName = "";
    tempTable2.Available = false;
    tempTable2.Item = { };
    for i = 1, table.getn(InventorySlots) do
        table.insert(tempTable2.Item, "");
    end
    
    return tempTable2;
end

---------------------------------------------------------------------------------
--
-- Each character on an account has an ID assigned to it that specifies its wardrobes
-- This function returns the ID associated with this character
-- 
---------------------------------------------------------------------------------
function Wardrobe_GetThisCharactersWardrobeID()

   WardrobeDebug("Looking up this character's wardrobe number...");
   
   -- look for this character in the wardrobe table
   charID = nil;
    for i = 1, table.getn(Wardrobe_Config) do
        if (Wardrobe_Config[i].PlayerName == UnitName("player")) then
            charID = i;
            break;
        end
    end
    
    -- if we didn't find this character, add us to the wardrobe table
    if (not charID) then
        Wardrobe_AddThisCharacterToWardrobeTable();
        charID = table.getn(Wardrobe_Config);
    end
    
    WardrobeDebug("This character's wardrobe number is: "..charID);

    -- remove any blank/unused outfits in case older versions of the AddOn
    -- were still storing unused outfits
    wardrobe_FilterUnusedoutfits();
    
    -- flag that we've already found / created this character's wardrobe entry
    Wardrobe_AlreadySetCharactersWardrobeID = true;
end


---------------------------------------------------------------------------------
--
-- Checks to see if we've already looked up the number associated with this character
-- If not, grab the number
-- 
---------------------------------------------------------------------------------
function Wardrobe_CheckForOurWardrobeID()
    if (not Wardrobe_AlreadySetCharactersWardrobeID) then
        Wardrobe_GetThisCharactersWardrobeID();
    end
end



--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              MAIN FUNCTIONS                                             --
--                                                                                            --
--============================================================================================--
--============================================================================================--


---------------------------------------------------------------------------------
--
-- Add the named outfit to our wardrobe
-- 
---------------------------------------------------------------------------------
function Wardrobe_AddNewOutfit(outfitName)   

     if (Wardrobe_Config.Enabled) then
    
       -- if we haven't already looked up our character's number
        Wardrobe_CheckForOurWardrobeID();

        -- make sure we don't already have an outfit with the same name
        if (Wardrobe_FoundOutfitName(outfitName)) then
            Print("An outfit with the same name already exists!  Please use a different name.", 1.0, 1.0, 0.5);   
            return;
        end

        WardrobeDebug("Trying to set this wardrobe as \""..outfitName.."\"");

        local outfitNum = Wardrobe_GetNextFreeOutfitSlot();

        -- if we found a free outfit slot
        if (outfitNum ~= 0) then
        
            -- store our current equipment in this outfit
            Wardrobe_StoreItemsInOutfit(outfitName, outfitNum, "added");

        -- otherwise we've used the maximum number of outfits
        else
            Print("You've used up all "..WARDROBE_NUM_OUTFITS.." of your outfits on this character.  Please delete one before creating another.", 1.0, 1.0, 0.5);
        end
    end
end


---------------------------------------------------------------------------------
--
-- Create and return the index of the next free outfit slot
-- 
---------------------------------------------------------------------------------
function Wardrobe_GetNextFreeOutfitSlot()

    -- find next unused outfit slot
    local outfitNum = 0;
    
    -- if we aren't already using our max number of outfits
    if (table.getn(Wardrobe_Config[charID].Outfit) < WARDROBE_NUM_OUTFITS) then
    
        -- add another outfit to the list and return its index
        table.insert(Wardrobe_Config[charID].Outfit, Wardrobe_CreateBlankOutfit());
        outfitNum = table.getn(Wardrobe_Config[charID].Outfit);
    end
        
    return outfitNum;    
end


---------------------------------------------------------------------------------
--
-- Store our currently equipped items in the specified outfit name
-- 
---------------------------------------------------------------------------------
function Wardrobe_StoreItemsInOutfit(outfitName, outfitNum, printMessage)

    -- store the name of this outfit
     Wardrobe_Config[charID].Outfit[outfitNum].OutfitName = outfitName;

    -- for each slot on our character's person (hands, feet, etc)
    for i = 1, table.getn(InventorySlots) do       
        Wardrobe_Config[charID].Outfit[outfitNum].Item[i] = Wardrobe_GetItemNameAtInventorySlotNumber(i);
        WardrobeDebug("    Setting slot "..InventorySlots[i].." = ["..Wardrobe_Config[charID].Outfit[outfitNum].Item[i].."]");
    end 

    -- all the items in this outfit are currently available in the player's inventory
    Wardrobe_Config[charID].Outfit[outfitNum].Available = true;

    Print("Outfit \""..outfitName.."\" "..printMessage..".", 1.0, 1.0, 0.5);

end


---------------------------------------------------------------------------------
--
-- Update an outfit
-- 
---------------------------------------------------------------------------------
function Wardrobe_UpdateOutfit(outfitName)

    if (Wardrobe_Config.Enabled) then
    
        -- if we haven't already looked up our character's number
        Wardrobe_CheckForOurWardrobeID();
             
        -- check to see if the wardrobe doesn't exist
        if (outfitName == nil or outfitName == "") then
            Print("Please enter an outfit name to update with your currently equipped items.",1.0,1.0,0.5);
        elseif (not Wardrobe_FoundOutfitName(outfitName)) then
            Print("That outfit name doesn't exist!  Please select an existing outfit to update with your currently equipped items.",1.0,1.0,0.5);
            UIErrorsFrame:AddMessage("Outfit name doesn't exist!", 1.0, 0.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
        else

            -- find the outfit to update
            for i = 1, table.getn(Wardrobe_Config[charID].Outfit) do

                -- if we found the outfit, store our equipment
                if (Wardrobe_Config[charID].Outfit[i].OutfitName == outfitName) then
                    Wardrobe_StoreItemsInOutfit(outfitName, i, "updated");
                end
            end
        end
    end
end




---------------------------------------------------------------------------------
--
-- Erase the named outfit
-- 
---------------------------------------------------------------------------------
function Wardrobe_EraseOutfit(outfitName)  

    if (Wardrobe_Config.Enabled) then
    
        -- if we haven't already looked up our character's number
        Wardrobe_CheckForOurWardrobeID();

        WardrobeDebug("Trying to delete outfit \""..outfitName.."\"");

        -- find the outfit to erase
        for i = 1, table.getn(Wardrobe_Config[charID].Outfit) do

            -- if we found the outfit
            if (Wardrobe_Config[charID].Outfit[i].OutfitName == outfitName) then

                -- remove the outfit
                table.remove(Wardrobe_Config[charID].Outfit, i);
                
                Wardrobe_RemoveAPopupButton(outfitName);

                if (not DoEraseAll) then
                    Print("Outfit \""..outfitName.."\" deleted.", 1.0, 1.0, 0.5);
                    Wardrobe_ListOutfits();
                    UIErrorsFrame:AddMessage("Outfit \""..outfitName.."\" deleted.", 0.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
                end
                return true;
            end
        end

        Print("Unable to find an outfit named \""..outfitName.."!\"", 1.0, 1.0, 0.5);
        UIErrorsFrame:AddMessage("Unable to find outfit!", 1.0, 0.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
        return false;
    end
end


---------------------------------------------------------------------------------
--
-- Erase all our outfits
-- 
---------------------------------------------------------------------------------
function Wardrobe_EraseAllOutfits()

    if (Wardrobe_Config.Enabled) then
    
        -- if we haven't already looked up our character's number
        Wardrobe_CheckForOurWardrobeID();

        -- remove all the popup buttons
        for i = 1, table.getn(Wardrobe_Config[charID].Outfit) do
           Wardrobe_RemoveAPopupButton(Wardrobe_Config[charID].Outfit[i].OutfitName);
        end
        
        -- delete all the outfits
        Wardrobe_Config[charID].Outfit = { };
        
        Print("All outfits deleted!", 1.0, 1.0, 0.5);
        UIErrorsFrame:AddMessage("All outfits deleted!", 0.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
    end
end


---------------------------------------------------------------------------------
--
-- Print a list of our outfits
-- 
---------------------------------------------------------------------------------
function Wardrobe_ListOutfits(var)

    if (Wardrobe_Config.Enabled) then
    
        -- if we haven't already looked up our character's number
        Wardrobe_CheckForOurWardrobeID();

        local foundOutfits = false;
        Print("Your current outfits are:", 1.0, 1.0, 0.5);

        -- for each outfit
        for i = 1, table.getn(Wardrobe_Config[charID].Outfit) do

            -- if it has a name
            if (Wardrobe_Config[charID].Outfit[i].OutfitName ~= "") then
                Print("    o "..Wardrobe_Config[charID].Outfit[i].OutfitName, 1.0, 1.0, 0.5);
                foundOutfits = true;

                -- if we asked for a detailed printout, show all the items
                if (var == "items") then
                    for j = 1, table.getn(InventorySlots) do    
                        if (Wardrobe_Config[charID].Outfit[i].Item[j] ~= "") then
                           Print("        ["..InventorySlots[j].." -> "..Wardrobe_Config[charID].Outfit[i].Item[j].."]", 1.0, 1.0, 0.5);
                        end
                    end           
                end
            end
        end

        if (not foundOutfits) then
            Print("  No outfits found!", 1.0, 1.0, 0.5);
        end
    end
end


---------------------------------------------------------------------------------
--
-- Wear an outfit
-- 
---------------------------------------------------------------------------------
function Wardrobe_WearOutfit(wardrobeName)

    if (Wardrobe_Config.Enabled) then
    
         -- if we haven't already looked up our character's number
         Wardrobe_CheckForOurWardrobeID();

        -- if the user didn't specify a wardrobe to wear
        if (not wardrobeName) then

            Print("Please specify an outfit to wear.", 1.0, 1.0, 0.5);
            return;

        -- else use the specified wardrobe
        else
            OutfitNumber = 0;
            for i = 1, table.getn(Wardrobe_Config[charID].Outfit) do
                WardrobeDebug("In WearOutfit, Looking at outfit #"..i.."  name = ["..Wardrobe_Config[charID].Outfit[i].OutfitName.."]");
                if (Wardrobe_Config[charID].Outfit[i].OutfitName == wardrobeName) then
                    OutfitNumber = i;
                    WardrobeDebug("Wardrobe_WearOutfit: Found outfit at #"..OutfitNumber);
                    break;
                end
            end

            if (OutfitNumber == 0) then
                Print("Unable to find \""..wardrobeName.."\" in your list of outfits!", 1.0, 1.0, 0.5);
                return;       
            end
        end

        WardrobeDebug("Switching to wardrobe \""..wardrobeName.."\"");

        -- this variable "freeBagSpacesUsed" lets us track which empty pack spaces we've
        -- already assigned an item to be put into.  we need to do this because when we remove
        -- items from our character and put them into our bags, the server takes time to actually
        -- move the item into the bag.  during this delay, we may be still removing items, and we
        -- may see a slot that LOOKS empty but really the server just hasn't gotten around to moving
        -- a previous item into the slot.  this variable lets us mark each empty slot once we've assigned
        -- an item to it so that we don't try to use the same empty slot for another item.
        local freeBagSpacesUsed = { };

        -- tracks how our switching is going.  if at any point we can't remove an item (bags are full, etc),
        -- this will get set to false
        local switchResult = true;

        -- for each slot on our character (hands, neck, head, feet, etc)
        for i = 1, table.getn(InventorySlots) do
            theSlotID = GetInventorySlotInfo(InventorySlots[i]);
            theItemName = Wardrobe_Config[charID].Outfit[OutfitNumber].Item[i];
            WardrobeDebug("Working on slot -> "..InventorySlots[i]);

            -- if we've set an item for this slot
            if (Wardrobe_Config[charID].Outfit[OutfitNumber].Item[i] ~= "") then

                -- if this item is different from what we're already wearing
                WardrobeDebug("   Comparing ["..theItemName.."] with ["..Wardrobe_GetItemNameAtInventorySlotNumber(i).."]");
                if (Wardrobe_Config[charID].Outfit[OutfitNumber].Item[i] ~=  Wardrobe_GetItemNameAtInventorySlotNumber(i)) then

                    WardrobeDebug("      Didn't match!  Switching out "..InventorySlots[i].." for ["..theItemName.."]");
                    
                    -- equip the item
                    if (not Wardrobe_Equip(theItemName, theSlotID)) then
                        Print("Warning: Unable to find item \""..theItemName.."\" in your bags!", 1.0, 0.0, 0.0);
                    end

                -- else we're already wearing it so no need to do anything
                else
                    WardrobeDebug("      Matched!  No need to switch out "..InventorySlots[i].." ["..theItemName.."]");
                end

            -- else this wardrobe doesn't use an item for this inventory slot (i.e. no gloves in this wardrobe)
            else

                -- if the inventory slot has an item equipped
                if (Wardrobe_GetItemNameAtInventorySlotNumber(i) ~= "") then

                    -- grab the inventory item and bag it
                    PickupInventoryItem(theSlotID);
                    result, freeBagSpacesUsed = BagItem(freeBagSpacesUsed);
                    WardrobeDebug("   Trying to remove "..InventorySlots[i].." ( slot ID #"..theSlotID..")");

                    -- if we failed to switch, this will let us know
                    switchResult = switchResult and result;
                end
            end
        end 

        -- if everything went OK
        if (switchResult) then
            Print("Wardrobe: Switched to outfit \""..wardrobeName..".\"", 1.0, 1.0, 0.5);
            Wardrobe_Current_Outfit = OutfitNumber;
        else
            Print("Wardrobe: Problems changing outfits.  Bags might be full.", 1.0, 1.0, 0.5);
        end
    end
end


---------------------------------------------------------------------------------
--
-- Rename an outfit
-- 
---------------------------------------------------------------------------------
function Wardrobe_RenameOutfit(oldName, newName)

    if (Wardrobe_Config.Enabled) then    

        -- check to see if the new name is already being used
        if (Wardrobe_FoundOutfitName(newName)) then
            Print("That outfit name already exists.  Please choose another name.",1.0,1.0,0.5);
            UIErrorsFrame:AddMessage("Outfit name already exists!", 1.0, 0.0, 0.0, 1.0, UIERRORS_HOLD_TIME);

        -- otherwise perform the rename
        else
            for i = 1, table.getn(Wardrobe_Config[charID].Outfit) do
                if (Wardrobe_Config[charID].Outfit[i].OutfitName == oldName) then
                    Wardrobe_Config[charID].Outfit[i].OutfitName = newName;
                    break;
                end
            end   
            UIErrorsFrame:AddMessage("Outfit renamed.", 0.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
            Print("Renamed outfit \""..oldName.."\" to \""..newName.."\"",1.0,1.0,0.5);
        end
    end
end



---------------------------------------------------------------------------------
--
-- Comparison function for sorting outfits
-- 
---------------------------------------------------------------------------------
function Wardrobe_SortOutfitCompare(outfit1, outfit2)
    if (outfit1.OutfitName < outfit2.OutfitName) then
        return true;
    else
        return false;
    end
end


---------------------------------------------------------------------------------
--
-- Sort the outfit alphabetically
-- 
---------------------------------------------------------------------------------
function Wardrobe_SortOutfits()
    table.sort(Wardrobe_Config[charID].Outfit, Wardrobe_SortOutfitCompare);
    UIErrorsFrame:AddMessage("Outfits sorted.", 0.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
    Print("Sorted outfits.",1.0,1.0,0.5);
end


---------------------------------------------------------------------------------
--
-- Update whether we have all the items for our outfits in our bags
-- 
---------------------------------------------------------------------------------
function Wardrobe_UpdateOutfitAvailability()

    if (Wardrobe_Config.Enabled and not Wardrobe_InCombat) then
    
        -- if we haven't already looked up our character's number
        Wardrobe_CheckForOurWardrobeID();

        WardrobeDebug("Wardrobe Availability:");
        
        local masterItemList = Wardrobe_BuildItemList();
    
        -- for each outfit
        for i = 1, table.getn(Wardrobe_Config[charID].Outfit) do

            -- if it has a name
            if (Wardrobe_Config[charID].Outfit[i].OutfitName ~= "") then

                local foundAllItems = true;

                -- for each item in the outfit
                for j = 1, table.getn(Wardrobe_Config[charID].Outfit[i].Item) do

                    local theItemName = Wardrobe_Config[charID].Outfit[i].Item[j];
                    if (theItemName ~= "") then
                    
                        local foundTheItem = false;
                        for k = 1, table.getn(masterItemList) do
                            if (theItemName == masterItemList[k]) then
                                foundTheItem = true;
                                break;
                            end
                        end
                        if (not foundTheItem) then 
                            foundAllItems = false;
                            break;
                        end
                        --if ((not Wardrobe_FindContainerItem(theItemName)) and (not Wardrobe_FindInventoryItem(theItemName))) then
                        --    foundAllItems = false;
                        --    break;
                        --end
                    end
                end

                -- if we found all items in our inventory
                Wardrobe_Config[charID].Outfit[i].Available = foundAllItems;
                WardrobeDebug("   Outfit \""..Wardrobe_Config[charID].Outfit[i].OutfitName.."\" -- found all items = "..tostring(foundAllItems));
            end
        end
    end
end


---------------------------------------------------------------------------------
--
-- Determine which outfit we're currently wearing
-- 
---------------------------------------------------------------------------------
function Wardrobe_DetermineActiveOutfit()

    WardrobeDebug("Wardrobe_DetermineActiveOutfit: Updating Active Outfit");
    Wardrobe_ActiveOutfit = 0;
    local foundOutfit = false;
    
    -- for each outfit
    for i = 1, table.getn(Wardrobe_Config[charID].Outfit) do
    
        WardrobeDebug("  Working on outfit "..i..": "..Wardrobe_Config[charID].Outfit[i].OutfitName);
        
        foundOutfit = true;
        
        -- for each slot on our character (hands, neck, head, feet, etc)
        for j = 1, table.getn(InventorySlots) do
        
            -- if this item is different from what we're already wearing
            WardrobeDebug("    Working on slot -> "..InventorySlots[j]);
            WardrobeDebug("       Comparing ["..Wardrobe_Config[charID].Outfit[i].Item[j].."] with ["..Wardrobe_GetItemNameAtInventorySlotNumber(j).."]");
            if (Wardrobe_Config[charID].Outfit[i].Item[j] ~=  Wardrobe_GetItemNameAtInventorySlotNumber(j)) then
                foundOutfit = false;
                break;
            end
        end
        
        if (foundOutfit) then
            Wardrobe_ActiveOutfit = i;
            break;
        end
     end
     
     WardrobeDebug("Wardrobe_DetermineActiveOutfit: Active Outfit = "..Wardrobe_ActiveOutfit);
     return Wardrobe_ActiveOutfit;     
end



--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              ITEM, BAG, TOOLTIP FUNCTIONS                                  --
--                                                                                            --
--============================================================================================--
--============================================================================================--



---------------------------------------------------------------------------------
--
-- Given the full text of a tooltip, return just the name of the item
-- 
---------------------------------------------------------------------------------
function Wardrobe_GetItemNameFromTooltip(itemTip)
    if (type(itemTip) ~= "table") then
        return itemTip;
    elseif (type(itemTip[1]) == "table") then
        return itemTip[1]["left"];
    else
        return "NIL";
    end
end


---------------------------------------------------------------------------------
--
-- Return the name of the item in the specified player's inventory slot number 
-- (1 = "headslot" etc - see InventorySlots above for a full list of the slots)
-- 
---------------------------------------------------------------------------------
function Wardrobe_GetItemNameAtInventorySlotNumber(slotNum)

    local itemName = "";
    
    -- get the id number for the item slot
    local id = GetInventorySlotInfo(InventorySlots[slotNum]);

    -- point the tooltip at this slot
    Wardrobe_MoneyToggle();
    local hasItem = WardrobeTooltip:SetInventoryItem("player", id);
    Wardrobe_MoneyToggle();
    if (hasItem) then
        itemName = Wardrobe_GetItemNameFromToolTip();
    end
    
    return itemName;
end


---------------------------------------------------------------------------------
--
-- Find the named item on the character's inventory (head slot, hand slot, etc -- not bags)
-- 
---------------------------------------------------------------------------------
function Wardrobe_FindInventoryItem(targetItemName)

    local foundItem = false;
    
    for i = 1, table.getn(InventorySlots) do
    
        if (Wardrobe_GetItemNameAtInventorySlotNumber(i) == targetItemName) then
            foundItem = true;
            break;
        end
    end
    
    return foundItem;
end


---------------------------------------------------------------------------------
--
-- Return the name of the item in the specified bag and slot
-- 
---------------------------------------------------------------------------------
function Wardrobe_GetItemNameInBagSlot(bagNum, slotNum)

    Wardrobe_MoneyToggle();
    WardrobeTooltip:SetBagItem(bagNum, slotNum);
    Wardrobe_MoneyToggle();
    itemName = Wardrobe_GetItemNameFromToolTip();
    
    return itemName;
end


---------------------------------------------------------------------------------
--
-- Return the name of the item that the tooltip is pointing at
-- 
---------------------------------------------------------------------------------
function Wardrobe_GetItemNameFromToolTip()

    local itemName = "";
    
    Wardrobe_MoneyToggle();
    ttext = getglobal("WardrobeTooltipTextLeft1");
    if (ttext and ttext:IsVisible() and ttext:GetText() ~= nil) then
        itemName = ttext:GetText();
    end
    Wardrobe_MoneyToggle();
    
    return itemName;
end


---------------------------------------------------------------------------------
--
-- Given the name of an item, find the bag and slot the item is in
-- 
---------------------------------------------------------------------------------
function Wardrobe_FindContainerItem(targetItemName)
    local foundBag = nil;
    local foundSlot = nil;
    
    --WardrobeDebug("    Looking in inventory for ["..targetItemName.."]");
    
    -- for each bag and slot
    for bag = 0, NUM_CONTAINER_FRAMES, 1 do
        local frame = getglobal("ContainerFrame"..bag);
        for slot = 1, GetContainerNumSlots(bag) do
        
            -- get the name of the item in this bag/slot
            bagItemName = Wardrobe_GetItemNameInBagSlot(bag, slot);
            --WardrobeDebug("         Comparing with "..bagItemName);
            
            -- if it matches what we're looking for
            if (targetItemName == bagItemName) then
               --WardrobeDebug("           FOUND in bag = "..bag.." slot = "..slot);
                foundBag  = bag;
                foundSlot = slot;
                break;
            end
        end
        if (foundBag and foundSlot) then
            break;
        end
    end
    
    return foundBag, foundSlot;
end


---------------------------------------------------------------------------------
--
-- Build a list of items in our bags and on our person
-- 
---------------------------------------------------------------------------------
function Wardrobe_BuildItemList()

    tempList = {};
    
    -- for each bag and slot
    for bag = 0, NUM_CONTAINER_FRAMES, 1 do
        local frame = getglobal("ContainerFrame"..bag);
        for slot = 1, GetContainerNumSlots(bag) do
        
            -- get the name of the item in this bag/slot
            local itemName = Wardrobe_GetItemNameInBagSlot(bag, slot);
            if (itemName ~= "") then
                table.insert(tempList, itemName);          
            end
        end
    end

    -- for each inventory item
    for i = 1, table.getn(InventorySlots) do   
        local itemName = Wardrobe_GetItemNameAtInventorySlotNumber(i);
        if (itemName ~= "") then
            table.insert(tempList, itemName);          
        end
    end

    --for i = 1, table.getn(tempList) do
    --    Print(i.." = "..tempList[i],0,1,0);
    --end
    
    return tempList;    
end



---------------------------------------------------------------------------------
--
-- Equip the specified item into the specified slot on our character (hands, chest, etc)
-- 
---------------------------------------------------------------------------------
function Wardrobe_Equip(itemName, equipSlot)

    -- if we're already holding something, bail
    if (CursorHasItem()) then
        return false;
    end
    
    -- find the bag and slot of this item
    local bag, slot = Wardrobe_FindContainerItem(itemName);
    if (bag and slot) then
        PickupContainerItem(bag, slot);
        if (equipSlot) then
            PickupInventoryItem(equipSlot);
        else
            AutoEquipCursorItem();
        end
        return true;
    end
    return false;
end


---------------------------------------------------------------------------------
--
-- Check to see if the specified outfit name is already being used
-- 
---------------------------------------------------------------------------------
function Wardrobe_FoundOutfitName(outfitName)
    local foundOutfit = false;
    
    for i = 1, table.getn(Wardrobe_Config[charID].Outfit) do
        if (Wardrobe_Config[charID].Outfit[i].OutfitName == outfitName) then
            foundOutfit = true;
            break;
        end
    end
    
    return foundOutfit;
end



---------------------------------------------------------------------------------
--
-- See if this bag / slot is being used or is free
-- 
---------------------------------------------------------------------------------
function Wardrobe_UsedThisSlot(freeBagSpacesUsed, theBag, theSlot)
    for i = 1, table.getn(freeBagSpacesUsed) do
        if (freeBagSpacesUsed[i][1] == theBag and freeBagSpacesUsed[i][2] == theSlot) then
            return true;
        end
    end
    return false;
end


---------------------------------------------------------------------------------
--
-- Put an item into a free bag slot
-- 
---------------------------------------------------------------------------------
function BagItem(freeBagSpacesUsed)

    -- for each bag and slot
    for theBag = 0, 4, 1 do
        local numSlot = GetContainerNumSlots(theBag);
        for theSlot = 1, numSlot, 1 do
        
            -- get info about the item here
            local texture, itemCount, locked = GetContainerItemInfo(theBag, theSlot);
            
            -- if we found nothing, add us to the list of free bag slots
            if (not texture and not Wardrobe_UsedThisSlot(freeBagSpacesUsed, theBag, theSlot)) then
                PickupContainerItem(theBag,theSlot);
                table.insert(freeBagSpacesUsed, {theBag, theSlot});
                return true, freeBagSpacesUsed;
            end
        end
    end
    
    AutoEquipCursorItem();
    
    return false, freeBagSpacesUsed;
end


--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              INTERFACE FUNCTIONS                                             --
--                                                                                            --
--============================================================================================--
--============================================================================================--


---------------------------------------------------------------------------------
--
-- Show the popup menu
-- 
---------------------------------------------------------------------------------
function Wardrobe_ShowMenu()

    -- if we haven't already looked up our character's number
    Wardrobe_CheckForOurWardrobeID();

    -- update the availability of our outfits
    Wardrobe_UpdateOutfitAvailability();

    -- find out if we're wearing one of our outfits
    local activeOutfitNum = Wardrobe_DetermineActiveOutfit();
    
    WardrobeDebug("Wardrobe_ShowMenu called!");
    
    -- set the title of the popup window
    Wardrobe_PopupTitle:SetText(UnitName("player").."'s "..WARDROBE_TEXT_MENU_TITLE);
    
    -- Set the text for the buttons while keeping track of how many
    -- buttons we actually need.
    local count = 0;
    for i = 1, table.getn(Wardrobe_Config[charID].Outfit) do
        if (Wardrobe_Config[charID].Outfit[i].OutfitName ~= "") then
            count = count + 1;
            local button = getglobal("Wardrobe_PopupButton"..count);
            button:SetText(Wardrobe_Config[charID].Outfit[i].OutfitName);
            
            -- color the buttons based on wardrobe availability
            if (i == activeOutfitNum) then
                button:SetTextColor(0.00, 1.00, 0.00);
            elseif (Wardrobe_Config[charID].Outfit[i].Available) then
                button:SetTextColor(0.28, 0.73, 0.28);
            else
                button:SetTextColor(0.70, 0.76, 0.65);
            end
            button.OutfitName = Wardrobe_Config[charID].Outfit[i].OutfitName;
            button:Show();
        end
    end
    
    -- add in the "add" and "delete" and "rename" and "sort" buttons
    count = count + 1;
    local button = getglobal("Wardrobe_PopupButton"..count);
    button:SetText("[Add New]");
    button.OutfitName = "[Add New]";
    button:SetTextColor(0.72, 0.51, 0.95);
    button:Show();
    count = count + 1;
    local button = getglobal("Wardrobe_PopupButton"..count);
    button:SetText("[Delete]");
    button.OutfitName = "[Delete]";
    button:SetTextColor(0.72, 0.51, 0.95);
    button:Show();
    count = count + 1;
    local button = getglobal("Wardrobe_PopupButton"..count);
    button:SetText("[Rename]");
    button.OutfitName = "[Rename]";
    button:SetTextColor(0.72, 0.51, 0.95);
    button:Show();
    count = count + 1;
    local button = getglobal("Wardrobe_PopupButton"..count);
    button:SetText("[Update]");
    button.OutfitName = "[Update]";
    button:SetTextColor(0.72, 0.51, 0.95);
    button:Show();
    count = count + 1;
    local button = getglobal("Wardrobe_PopupButton"..count);
    button:SetText("[Sort]");
    button.OutfitName = "[Sort]";
    button:SetTextColor(0.72, 0.51, 0.95);
    button:Show();
    
    -- Set the width for the menu.
    local width = Wardrobe_PopupTitle:GetWidth();
    for i = 1, count, 1 do
        local button = getglobal("Wardrobe_PopupButton"..i);
        local w = button:GetTextWidth();
        if (w > width) then
            width = w;
        end
    end
    Wardrobe_Popup:SetWidth(width + 2 * UNITPOPUP_BORDER_WIDTH);

    -- By default, the width of the button is set to the width of the text
    -- on the button.  Set the width of each button to the width of the
    -- menu so that you can still click on it without being directly
    -- over the text.
    for i = 1, count, 1 do
        local button = getglobal("Wardrobe_PopupButton"..i);
        button:SetWidth(width);
    end

    -- Hide the buttons we don't need.
    for i = count + 1, WARDROBE_NUM_OUTFITS + WARDROBE_NUM_POPUP_FUNCTION_BUTTONS do
        local button = getglobal("Wardrobe_PopupButton"..i);
        button:Hide();
    end
    
    -- Set the height for the menu.
    Wardrobe_Popup:SetHeight(UNITPOPUP_TITLE_HEIGHT + (count * (UNITPOPUP_BUTTON_HEIGHT - 3)) + (3 * UNITPOPUP_BORDER_HEIGHT));

    -- Show the menu.
    Wardrobe_Popup:Show();
end


---------------------------------------------------------------------------------
--
-- Handle when a user clicks on a wardrobe button
-- 
---------------------------------------------------------------------------------
function Wardrobe_ButtonClick()
    local Wardrobe_PopupWindowHeader = getglobal("WardrobeNamePopupText");

    if (this.OutfitName == "[Add New]") then
        Wardrobe_PopupWindowHeader:SetText("New Outfit Name");
        Wardrobe_PopupFunction = this.OutfitName;
        WardrobeNamePopup:Show();
    elseif (this.OutfitName == "[Delete]") then
        Wardrobe_PopupWindowHeader:SetText("Outfit To Delete");
        Wardrobe_PopupFunction = this.OutfitName;
        WardrobeNamePopup:Show();
    elseif (this.OutfitName == "[Rename]") then
        Wardrobe_PopupWindowHeader:SetText("Outfit To Rename");
        Wardrobe_PopupFunction = "[Rename]";
        WardrobeNamePopup:Show();
    elseif (this.OutfitName == "[Sort]") then
        Wardrobe_SortOutfits();
    elseif (this.OutfitName == "[Update]") then
        Wardrobe_PopupWindowHeader:SetText("Outfit To Update");
        Wardrobe_PopupFunction = "[Update]";
        WardrobeNamePopup:Show();
    else
        Wardrobe_WearOutfit(this.OutfitName);
    end
        
    Wardrobe_Popup:Hide();
end


---------------------------------------------------------------------------------
--
-- Confirm a popup menu
-- 
---------------------------------------------------------------------------------
function Wardrobe_PopupConfirm()

    WardrobeNamePopup:Hide();
    local Wardrobe_PopupWindowHeader = getglobal("WardrobeNamePopupText");

    if (Wardrobe_PopupFunction == "[Add New]") then
        Wardrobe_AddNewOutfit(WardrobeNamePopupEditBox:GetText());
        Wardrobe_PopupFunction = "";
    elseif (Wardrobe_PopupFunction == "[Delete]") then
        Wardrobe_EraseOutfit(WardrobeNamePopupEditBox:GetText());
        Wardrobe_PopupFunction = "";
    elseif (Wardrobe_PopupFunction == "[Rename]") then
        if (not Wardrobe_FoundOutfitName(WardrobeNamePopupEditBox:GetText()) or WardrobeNamePopupEditBox:GetText() == "") then
            Print("That outfit doesn't exist.  Please enter an existing outfit to rename.",1.0,1.0,0.5);
            UIErrorsFrame:AddMessage("Outfit name doesn't exist!", 0.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
            Wardrobe_PopupFunction = "";
        else
            Wardrobe_Rename_OldName = WardrobeNamePopupEditBox:GetText();
            Wardrobe_PopupFunction = "[Rename2]"
            Wardrobe_PopupWindowHeader:SetText("New Name");
            WardrobeNamePopup:Show();
        end
    elseif (Wardrobe_PopupFunction == "[Rename2]") then
        Wardrobe_RenameOutfit(Wardrobe_Rename_OldName, WardrobeNamePopupEditBox:GetText());
        Wardrobe_PopupFunction = "";
    elseif (Wardrobe_PopupFunction == "[Update]") then
        Wardrobe_UpdateOutfit(WardrobeNamePopupEditBox:GetText());
        Wardrobe_PopupFunction = "";
    end
end


---------------------------------------------------------------------------------
--
-- Cancel a popup menu
-- 
---------------------------------------------------------------------------------
function Wardrobe_PopupCancel()
    WardrobeNamePopup:Hide();
end


---------------------------------------------------------------------------------
--
-- Remove a button from the popup menu list
-- 
---------------------------------------------------------------------------------
function Wardrobe_RemoveAPopupButton(wardrobeName)
    for i = 1, WARDROBE_NUM_OUTFITS + WARDROBE_NUM_POPUP_FUNCTION_BUTTONS do
        local button = getglobal("Wardrobe_PopupButton"..i);
        if (wardrobeName == button:GetText()) then
            button:SetText("");
            button.OutfitName = "";
            button:SetTextColor(1.0, 1.0, 1.0);
            button:Hide();
            break;
        end
    end
end


---------------------------------------------------------------------------------
--
-- Start dragging the wardrobe button
-- 
---------------------------------------------------------------------------------
function Wardrobe_OnDragStart()
    if (not Wardrobe_DragLock) then
        Wardrobe_IconFrame:StartMoving()
        Wardrobe_BeingDragged = true;
    end
end


---------------------------------------------------------------------------------
--
-- Stop dragging the wardrobe button
-- 
---------------------------------------------------------------------------------
function Wardrobe_OnDragStop()
    if(Wardrobe_BeingDragged) then
        Wardrobe_IconFrame:StopMovingOrSizing()
        Wardrobe_BeingDragged = false;
        local x,y = this:GetCenter();
        local px,py = this:GetParent():GetCenter();
        local ox = x-px;
        local oy = y-py;
        Wardrobe_Config.xOffset = ox;
        Wardrobe_Config.yOffset = oy;
        
--[[                
        local popupWidth = Wardrobe_Popup:GetWidth();
        Print("x = "..x.." popupWidth = "..popupWidth);
        if (x - popupWidth > popupWidth) then
            Wardrobe_Popup:SetPoint("TOPRIGHT", "Wardrobe_IconFrame", "TOPLEFT", 1, 1);
        else
            Wardrobe_Popup:SetPoint("TOPLEFT", Wardrobe_Popup:GetParent():GetName(), "TOPRIGHT", 1, 1);        
        end
]]--
    end
end


---------------------------------------------------------------------------------
--
-- Toggle whether the wardrobe button can be moved or not
-- 
---------------------------------------------------------------------------------
function Wardrobe_ToggleLockButton(toggle)
    Wardrobe_DragLock = toggle;
    if (Wardrobe_DragLock) then
        Print("Wardrobe button locked.  To reposition, use /wardrobe unlock", 1.0, 1.0, 0.5);
    else
        Print("Wardrobe button unlocked.  You may reposition the wardrobe button.  To lock the button in place, use /wardrobe lock", 1.0, 1.0, 0.5);
    end
end


---------------------------------------------------------------------------------
--
-- Move the wardrobe button to the specified location
-- 
---------------------------------------------------------------------------------
function Wardrobe_ButtonSetLocation()
    Wardrobe_IconFrame:SetPoint("CENTER", Wardrobe_IconFrame:GetParent():GetName(), "CENTER", Wardrobe_Config.xOffset, Wardrobe_Config.yOffset);
end


---------------------------------------------------------------------------------
--
-- Register the wardrobe button 
-- 
---------------------------------------------------------------------------------
function Wardrobe_ButtonOnLoad()
    this:RegisterForDrag("LeftButton");
end



--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              UTILITY FUNCTIONS                                             --
--                                                                                            --
--============================================================================================--
--============================================================================================--


---------------------------------------------------------------------------------
--
-- Save our settings
-- 
---------------------------------------------------------------------------------
function Wardrobe_SaveSettings()
    RegisterForSave("Wardrobe_Config");
end


-----------------------------------------------------------------------------------
--
-- Toggle the plugin on and off
--
-----------------------------------------------------------------------------------
function Wardrobe_Toggle(toggle)
    if (toggle == 1) then
        --Print("Wardrobe Enabled.", 1.0, 1.0, 0.5);
        Wardrobe_Config.Enabled = true;
        Wardrobe_IconFrame:Show();
    else
        --Print("Wardrobe Disabled.", 1.0, 1.0, 0.5);
        Wardrobe_Config.Enabled = false;
        Wardrobe_IconFrame:Hide();
    end
end


-----------------------------------------------------------------------------------
--
-- Nifty little function to view any lua object as text
--
-----------------------------------------------------------------------------------
function asText(obj)

    visitRef = {}
    visitRef.n = 0

    asTxRecur = function(obj, asIndex)
        if type(obj) == "table" then
            if visitRef[obj] then
                return "@"..visitRef[obj]
            end
            visitRef.n = visitRef.n +1
            visitRef[obj] = visitRef.n

            local begBrac, endBrac
            if asIndex then
                begBrac, endBrac = "[{", "}]"
            else
                begBrac, endBrac = "{", "}"
            end
            local t = begBrac
            local k, v = nil, nil
            repeat
                k, v = next(obj, k)
                if k ~= nil then
                    if t > begBrac then
                        t = t..", "
                    end
                    t = t..asTxRecur(k, 1).."="..asTxRecur(v)
                end
            until k == nil
            return t..endBrac
        else
            if asIndex then
                -- we're on the left side of an "="
                if type(obj) == "string" then
                    return obj
                else
                    return "["..obj.."]"
                end
            else
                -- we're on the right side of an "="
                if type(obj) == "string" then
                    return '"'..obj..'"'
                else
                    return tostring(obj)
                end
            end
        end
    end -- asTxRecur

    return asTxRecur(obj)
end -- asText


--============================================================================================--
--============================================================================================--
--                                                                                            --
--                              DEBUG FUNCTIONS                                               --
--                                                                                            --
--============================================================================================--
--============================================================================================--


-----------------------------------------------------------------------------------
--
-- Print out a debug statement if the WARDROBE_DEBUG flag is set
--
-----------------------------------------------------------------------------------
function WardrobeDebug(theMsg)
    if (WARDROBE_DEBUG) then
        ChatFrame1:AddMessage(theMsg, 1.0, 1.0, 0.7);
    end
end


---------------------------------------------------------------------------------
--
-- Toggle debug output
-- 
---------------------------------------------------------------------------------
function Wardrobe_ToggleDebug()
    WARDROBE_DEBUG = not WARDROBE_DEBUG;
    if (WARDROBE_DEBUG) then
        Print("Wardrobe: Debug ON",1.0,1.0,0.5);
    else
        Print("Wardrobe: Debug OFF",1.0,1.0,0.5);
    end    
end


---------------------------------------------------------------------------------
--
-- Debug routine to print the current state of the plugin
-- 
---------------------------------------------------------------------------------
function Wardrobe_DumpDebugReport()

    WardrobeDebug("Wardrobe_DumpDebugReport: Character's wardrobe database:");
    for wardrobeNum = 1, WARDROBE_NUM_OUTFITS do
        for i = 1, table.getn(InventorySlots) do
            theWardrobeItem = Wardrobe_Config[charID].Wardrobe[wardrobeNum].Item[i];
            WardrobeDebug("  Set #"..wardrobeNum.." -> "..InventorySlots[i].." = "..theWardrobeItem);
        end
    end
end


