akz_creatures = {}; -- [saved] mob info
akz_profile = {};   -- [saved] character profile
akz_fishing = {};   -- [saved] fishing data
akz_objects = {};   -- [saved] world object data
akz_prices = {};    -- [saved] auction tracking

akz_uploaded = 0;   -- [saved] track data uploads

akz_kills = {};       -- [not saved] track kill info while running
akz_lastobjtime = 0;  -- [not saved] object tracking
akz_lastobject = "";  -- [not saved] object tracking

akz_lastauctionupdatetime = 0; -- [not saved] auction timing
akz_WrVersion = 1.52; -- [not saved] this file version

-- array of inventory slot names
akz_slots = {
  "HeadSlot",          -- 1
  "NeckSlot",          -- 2
  "ShoulderSlot",      -- 3
  "ShirtSlot",         -- 4
  "ChestSlot",         -- 5
  "WaistSlot",         -- 6
  "LegsSlot",          -- 7
  "FeetSlot",          -- 8
  "WristSlot",         -- 9
  "HandsSlot",         -- 10
  "Finger0Slot",       -- 11
  "Finger1Slot",       -- 12
  "Trinket0Slot",      -- 13
  "Trinket1Slot",      -- 14
  "BackSlot",          -- 15
  "MainHandSlot",      -- 16
  "SecondaryHandSlot", -- 17
  "RangedSlot",        -- 18
  "TabardSlot",        -- 19
};

function WrFrame_OnLoad()
  this:RegisterEvent("LOOT_OPENED");
  this:RegisterEvent("PLAYER_LEAVING_WORLD");
  this:RegisterEvent("PLAYER_ENTERING_WORLD");
  this:RegisterEvent("UNIT_LEVEL");
  this:RegisterEvent("PLAYER_GUILD_UPDATE");
  this:RegisterEvent("UNIT_NAME_UPDATE");
  this:RegisterEvent("UNIT_INVENTORY_CHANGED");
  this:RegisterEvent("PLAYER_TARGET_CHANGED");
  this:RegisterEvent("ZONE_CHANGED");
  this:RegisterEvent("VARIABLES_LOADED");
  this:RegisterEvent("MERCHANT_UPDATE");
  this:RegisterEvent("MERCHANT_SHOW");
  this:RegisterEvent("SKILL_LINES_CHANGED");
  this:RegisterEvent("CHARACTER_POINTS_CHANGED");
  this:RegisterEvent("CHAT_MSG_SPELL_SELF_BUFF");
  this:RegisterEvent("AUCTION_ITEM_LIST_UPDATE");

  RegisterForSave("akz_creatures");
  RegisterForSave("akz_fishing");
  RegisterForSave("akz_profile");
  RegisterForSave("akz_uploaded");
  RegisterForSave("akz_objects");
  RegisterForSave("akz_prices");

  SLASH_WLOC1 = "/wloc";  -- /wloc command spits out the zone location of the player
  SlashCmdList["WLOC"] = function()
			    akz_PrintLoc("player");
			 end
  SLASH_WVER1 = "/wver"; -- /wver shows the version of this file.  Used to verify load.
  SlashCmdList["WVER"] = function()
			    akz_ShowVersion(akz_WrVersion);
			 end

  
  DEFAULT_CHAT_FRAME:AddMessage("WowReader Loaded");
end

function WrFrame_OnEvent(event, arg1)
  if ( event == "LOOT_OPENED" ) then
    akz_HandleLoot();
    return;
  end

  if ( event == "PLAYER_ENTERING_WORLD" ) then
    akz_UpdatePlayerInfo();    
    akz_GetSkills();
    akz_UpdateInventory();
    return;
  end
    
  if ( event == "UNIT_INVENTORY_CHANGED" ) then
    akz_UpdateInventory();
    akz_UpdatePlayerInfo();    
    return;
  end
    
  if ( event == "UNIT_LEVEL" ) then
    if ( arg1 == "player" ) then
       akz_UpdatePlayerInfo();
    end
    return;
  end

  if ( event == "PLAYER_GUILD_UPDATE" ) then
    akz_profile["guild"], akz_profile["gtitle"], akz_profile["grank"] = GetGuildInfo("player");
    return;
  end

  if ( event == "UNIT_NAME_UPDATE" ) then
    akz_profile["name"] = UnitName("player");
    return;
  end

  if ( event == "PLAYER_LEAVING_WORLD" ) then
    return;
  end
  
  if ( event == "PLAYER_TARGET_CHANGED" ) then
    if ( UnitExists("target") ) then
      if ( not UnitIsPlayer("target") and not UnitIsDeadOrGhost("target") and not UnitPlayerControlled("target")) then
        if ((UnitName("target") == "Doomguard") or (UnitName("target") == "Infernal")) then
          return;
        end
        akz_AddCreature(UnitName("target"));
        akz_AddLoc(UnitName("target"), akz_GetCurZoneLoc("player"));
        akz_AddLevel(UnitName("target"), UnitLevel("target"));
      end
    end
    return;
  end
  
  if ( (event == "MERCHANT_UPDATE") or (event == "MERCHANT_SHOW") ) then
    akz_DoMerchant();
    return;
  end

  if ( (event == "SKILL_LINES_CHANGED") or (event == "CHARACTER_POINTS_CHANGED") ) then
    akz_GetSkills();
    akz_UpdateTalents();
    return;
  end
  
  if ( event == "CHAT_MSG_SPELL_SELF_BUFF" ) then
    local num, num2, skill, opened = string.find(arg1, 'You perform (.-) on (.*)');
    if (opened) then
      if (skill == "Mining" or skill == "Herb Gathering" or skill == "Opening") then
        akz_lastobject = string.sub(opened, 0, string.len(opened) - 1);
        akz_lastobjtime = GetTime();
      end
    end
    return;
  end
  
  if ( (event == "AUCTION_ITEM_LIST_UPDATE") ) then
    akz_GetCurrentBids();
    return;
  end;
  
  if ( event == "VARIABLES_LOADED" ) then
    if (akz_uploaded == 1) then
      akz_creatures = {};
      akz_fishing = {};
      akz_objects = {};
      akz_prices = {};
      akz_uploaded = 0;
    end
    akz_profile = {};
    akz_prices = {};
  end
end


function akz_UpdatePlayerInfo()
    akz_profile["name"] = UnitName("player");
    akz_profile["race"] = UnitRace("player");
    akz_profile["class"] = UnitClass("player");
    akz_profile["level"] = UnitLevel("player");
    akz_profile["guild"], akz_profile["gtitle"], akz_profile["grank"] = GetGuildInfo("player");
    akz_profile["server"] = GetCVar("realmName");
    akz_profile["talentpts"],akz_profile["skillpts"] = UnitCharacterPoints("player");
    
    akz_profile["str"] = akz_GetBaseStat(1);
    akz_profile["agi"] = akz_GetBaseStat(2);
    akz_profile["sta"] = akz_GetBaseStat(3);
    akz_profile["intel"] = akz_GetBaseStat(4);
    akz_profile["spirit"] = akz_GetBaseStat(5);
   
    akz_profile["arcane_resist"] = UnitResistance("player",6);
    akz_profile["fire_resist"] = UnitResistance("player",2);
    akz_profile["nature_resist"] = UnitResistance("player",3);
    akz_profile["frost_resist"] = UnitResistance("player",4);
    akz_profile["shadow_resist"] = UnitResistance("player",5);

    akz_profile["health2"] = akz_CalcBaseHp();
    akz_profile["mana2"] = akz_CalcBaseMana();
    akz_profile["attack"] = UnitAttackBothHands("player");

    local RangeSpeeed, RangeMinDMG,RangeMaxDMG = UnitRangedDamage("player");
    akz_profile["ranged_damage"] = max(floor(RangeMinDMG),1) .. ":" .. max(ceil(RangeMaxDMG),1);
    local base, pos, neg = UnitRangedAttackPower("player");
    akz_profile["ranged_power"] = base;
    local base, pos, neg = UnitRangedAttack("player");
    akz_profile["ranged"] = base;

    local minDamage;
    local maxDamage; 
    local minOffHandDamage;
    local maxOffHandDamage; 
    local physicalBonusPos;
    local physicalBonusNeg;
    local percent;
    
    minDamage, maxDamage, minOffHandDamage, maxOffHandDamage, physicalBonusPos, physicalBonusNeg, percent = UnitDamage("player");
    
    local baseDamage = (minDamage + maxDamage) * 0.5;
    local fullDamage = (baseDamage + physicalBonusPos + physicalBonusNeg) * percent;
    
    local totalBonus = (fullDamage - baseDamage);
    local displayMin = max(floor(minDamage + totalBonus),1);
    local displayMax = max(ceil(maxDamage + totalBonus),1);
    
    local baseArm, effectiveArmor, armor, positiveArm, negativeArm = UnitArmor("player");
    
    akz_profile["damagerange"] = displayMin .. ":" .. displayMax;
    akz_profile["atk_power"] = UnitAttackPower("player");
    akz_profile["defense"] = UnitDefense("player");
    akz_profile["armour"] = effectiveArmor;
    
    akz_UpdateTalents();
    
    akz_profile["coin"] = GetMoney();
end

function akz_UpdateTalents() 
  akz_profile["talents"] = {};
  MAXTREES = 3;
  MAXPERTREE = 20;
  for i = 1, MAXTREES do
    akz_profile["talents"][i] = {};
    for j = 1, MAXPERTREE do
      local name, iconTexture, tier, column, rank, maxRank, isExceptional, meetsPrereq = GetTalentInfo(i, j);
      if (name) then
         table.insert(akz_profile["talents"][i], rank);
      else 
         akz_profile["talents"][i][j] = nil;
      end
    end
  end
end

function akz_UpdateInventory()
  local itemid;
  local enchant;
  local subid;
  local itemname;
  local link;
  
  for index,slot in akz_slots do
    link = GetInventoryItemLink("player", index)
    if (link) then
      itemid, enchant, subid, itemnane = akz_GetItemInfoFromLink(link);
      akz_profile[slot] = itemid .. ":" .. subid .. ":" .. enchant;
    else
      akz_profile[slot] = nil;
    end
  end
end

function akz_GetSkills ()
   akz_profile["skills"] = {};
   for i = 1,GetNumSkillLines() ,1 do
      local skillName, header, isExpanded, skillRank, numTempPoints, skillModifier, skillMaxRank, isAbandonable, stepCost, rankCost, minLevel, skillCostType = GetSkillLineInfo(i);
      if (skillName) then
         table.insert(akz_profile["skills"],skillName .. "^" .. skillRank .. "^" .. skillMaxRank);
      else
         akz_profile["skills"][i] = nil;
      end
   end
end

function akz_AddCreature(name)
  if (name == "Unknown Being") then
    return;
  end

  if (not akz_creatures[name]) then
    akz_creatures[name] = { };
    akz_creatures[name]["lvlmin"] = 0;
    akz_creatures[name]["lvlmax"] = 0;
    akz_creatures[name]["kills"] = 0;
  end
end

function akz_AddObject(name)
  if (not akz_objects[name]) then
    akz_objects[name] = { };
    akz_objects[name]["opened"] = 1;
  else
    akz_objects[name]["opened"] = akz_objects[name]["opened"] + 1;
  end
end

function akz_AddLoc(name, thiszone, locx, locy)
  if (not akz_creatures[name]) then
    return;
  end
  
  local tx, ty = akz_RoundLoc(locx, locy);
  if (((locx == 0) and (locy == 0)) or (thiszone == 0) or (thiszone == 1000) or (thiszone == 2000))  then
    local zt = GetZoneText();
    
    if (not zt) then
      return
    end
    
    if (not akz_creatures[name]["instances"]) then
      akz_creatures[name]["instances"] = { };
    end
    for index,inst in akz_creatures[name]["instances"] do
      if (inst==zt) then
        return;
      end
    end
    table.insert(akz_creatures[name]["instances"], zt);
    return;
  end
  
  if (not akz_creatures[name]["locs"]) then
    akz_creatures[name]["locs"] = { };
  end
  
  for index,locinfo in akz_creatures[name]["locs"] do
    if ( (locinfo["zone"]==thiszone) and (locinfo["x"]==tx) and (locinfo["y"]==ty) ) then
      return;
    end
  end
  
  local addloc = { ["x"]=tx, ["y"]=ty, ["zone"]=thiszone };
  table.insert(akz_creatures[name]["locs"], addloc);
end

function akz_AddObjLoc(name, thiszone, locx, locy)
  if (not akz_objects[name]) then
    return;
  end
  
  local tx, ty = akz_RoundLoc(locx, locy);
  if (((locx == 0) and (locy == 0)) or (thiszone == 0) or (thiszone == 1000) or (thiszone == 2000))  then
    local zt = GetZoneText();
    
    if (not zt) then
      return
    end
    
    if (not akz_objects[name]["instances"]) then
      akz_objects[name]["instances"] = { };
    end
    for index,inst in akz_objects[name]["instances"] do
      if (inst==zt) then
        return;
      end
    end
    table.insert(akz_objects[name]["instances"], zt);
    return;
  end
  
  if (not akz_objects[name]["locs"]) then
    akz_objects[name]["locs"] = { };
  end
  
  for index,locinfo in akz_objects[name]["locs"] do
    if ( (locinfo["zone"]==thiszone) and (locinfo["x"]==tx) and (locinfo["y"]==ty) ) then
      return;
    end
  end
  
  local addloc = { ["x"]=tx, ["y"]=ty, ["zone"]=thiszone };
  table.insert(akz_objects[name]["locs"], addloc);
end


function akz_AddObjLoot(name, itemid)
  if (not akz_objects[name]) then
    return;
  end

  if (not akz_objects[name]["items"]) then
    akz_objects[name]["items"] = { };
  end

  if (not akz_objects[name]["items"][itemid]) then
    akz_objects[name]["items"][itemid] = 1;
  else 
    akz_objects[name]["items"][itemid] = akz_objects[name]["items"][itemid] + 1;
  end
end

function akz_AddLevel(name, level)
  if (not akz_creatures[name]) then
    return;
  end
  
  if (level > akz_creatures[name]["lvlmax"]) then
    akz_creatures[name]["lvlmax"] = level;
  end
  
  if ((level < akz_creatures[name]["lvlmin"]) or (akz_creatures[name]["lvlmin"] == 0)) then
    akz_creatures[name]["lvlmin"] = level;
  end
end

function akz_CheckKill(name)
  if (not akz_creatures[name]) then
    return;
  end
  
  zn = GetZoneText();
--  tx = UnitFacing("target");
  local thiskill = { ["zone"]=zn };
  
  if (not akz_kills[name]) then
    akz_kills[name] = { };
    table.insert(akz_kills[name], thiskill);
    akz_creatures[name]["kills"] = akz_creatures[name]["kills"] + 1;
    return true;
  end
  
--  for index,killinfo in akz_kills[name] do
--    if ( (killinfo["zone"]==zn) and (killinfo["x"]==tx) ) then
--      return false;
--    end
--  end

  table.insert(akz_kills[name], thiskill);
  akz_creatures[name]["kills"] = akz_creatures[name]["kills"] + 1;

  return true;
end

function akz_HandleLoot()
  local itemid;
  local enchant;
  local subid;
  local itemname;
  
  if (IsFishingLoot()) then
    for index = 1, GetNumLootItems(), 1 do
      if (LootSlotIsItem(index)) then
        itemid, enchant, subid, itemname = akz_GetItemInfoFromLink(GetLootSlotLink(index));
        akz_AddFishLoot(GetMinimapZoneText(), itemid);
      end
    end
    return
  end

  if (not UnitName("target") and not UnitIsDead("target")) then
    if (GetTime() - 3 < akz_lastobjtime) then
      akz_AddObject(akz_lastobject);
      akz_AddObjLoc(akz_lastobject, akz_GetCurZoneLoc("player"));

      for index = 1, GetNumLootItems(), 1 do
        if (LootSlotIsItem(index)) then
          itemid, enchant, subid, itemname = akz_GetItemInfoFromLink(GetLootSlotLink(index));
          akz_AddObjLoot(akz_lastobject, itemid);
        end
      end

    end
  end

  if (not UnitName("target")) then
    return
  end

  if (not UnitIsDead("target")) then
    return
  end

  if (UnitIsPlayer("target")) then
    return
  end

  if (akz_CheckKill(UnitName("target"))) then
    for index = 1, GetNumLootItems(), 1 do
      if (LootSlotIsItem(index)) then
        itemid, enchant, subid, itemname = akz_GetItemInfoFromLink(GetLootSlotLink(index));
        akz_AddLoot(UnitName("target"), itemid);
      end
    end
  end
end

function akz_AddFishLoot(fishzone, itemid)
  if (akz_fishing[fishzone]) then
    for index,id in akz_fishing[fishzone] do
      if (id == itemid) then
        return
      end
    end
    table.insert(akz_fishing[fishzone], itemid);
  else
    akz_fishing[fishzone] = {};
    table.insert(akz_fishing[fishzone], itemid);
  end
end

function akz_AddLoot(name, itemid)
  if (not akz_creatures[name]) then
    return;
  end

  if (not akz_creatures[name]["items"]) then
    akz_creatures[name]["items"] = { };
  end

  if (not akz_creatures[name]["items"][itemid]) then
    akz_creatures[name]["items"][itemid] = 1;
  else 
    akz_creatures[name]["items"][itemid] = akz_creatures[name]["items"][itemid] + 1;
  end
end

function akz_DoMerchant()
  local itemid;
  local enchant;
  local subid;
  local itemname;
  local mlink;
  
  for index = 1, GetMerchantNumItems(), 1 do
    mlink = GetMerchantItemLink(index);
    if (mlink) then
      itemid, enchant, subid, itemname = akz_GetItemInfoFromLink(mlink);
      akz_AddLoot(UnitName("NPC"), itemid);
    end
  end
end


-- debugging
function WrFrame_Show()
  local field = getglobal("WR_edit1");
  local text = "";

  if (field) then
--[[
    for zone,items in akz_fishing do
      for index,item in items do
        text = text .. string.format("\n fishing: %s - %d", zone, item);
      end
    end

    field:SetText(text);
    field:SetFocus();
]]--
  end
end

-- utils
function akz_GetItemInfoFromLink(link)
  for itemid, enchant, subid, itemname in string.gfind(link, "|c%x+|Hitem:(%d+):(%d+):(%d+):%d+|h%[(.-)%]|h|r") do
    return itemid, enchant, subid, itemname;
  end
end

function akz_RoundLoc(locx, locy)
  local x, y;
  x = string.format("%d", locx * 100);
  y = string.format("%d", locy * 100);
  x = x + 0;
  y = y + 0;
  return x, y;
end

function akz_GetBaseStat(i)
  local stat;
  local effectiveStat;
  local posBuff;
  local negBuff;
  stat, effectiveStat, posBuff, negBuff = UnitStat("player", i);
  return stat - posBuff - negBuff;
end

function akz_CalcBaseHp()
  local stat, effectiveStat, posBuff, negBuff = UnitStat("player", 3); -- sta
  local bhp = UnitHealthMax("player");
  
  local stafrombuff = posBuff + negBuff;
  local hpfrombuff = stafrombuff * 10;
  
  if (UnitRace("player") == "Tauren") then  -- tauren racial 5% bonus
    hpfrombuff = string.format("%d", hpfrombuff * 1.05);
  end
  
  return bhp - hpfrombuff;
end

function akz_CalcBaseMana()
  local stat, effectiveStat, posBuff, negBuff = UnitStat("player", 4); -- int
  local bm = UnitManaMax("player");
  
  local intfrombuff = posBuff + negBuff;
  local manafrombuff = intfrombuff * 15;
  
  if (UnitClass("player") == "Priest") then -- priest talent 2-10% bonus
    if (akz_profile["talents"]) then
      if (akz_profile["talents"][1][10]) then
        if (akz_profile["talents"][1][10] > 0) then
          manafrombuff = string.format("%d", manafrombuff * (((akz_profile["talents"][1][10] * 2) * 0.01) + 1));
        end
      end
    end
  end
  
  if (UnitClass("player") == "Shaman") then -- shaman 1-5% bonus
    if (akz_profile["talents"]) then
      if (akz_profile["talents"][2][1]) then
        if (akz_profile["talents"][2][1] > 0) then
          manafrombuff = string.format("%d", manafrombuff * ((akz_profile["talents"][2][1] * 0.01) + 1));
        end
      end
    end
  end
  
  return bm - manafrombuff;
end

function akz_GetCurZoneLoc(t)
  SetMapToCurrentZone();
  local cont = GetCurrentMapContinent();
  local zone = GetCurrentMapZone();
  
  cont = cont * 1000;
  zone = zone + cont;
  return zone, GetPlayerMapPosition(t);
end

function akz_PrintLoc (t)
   local x,y = GetPlayerMapPosition(t)
   x,y = akz_RoundLoc(x,y);
   if (x == 0 and y == 0) then
      DEFAULT_CHAT_FRAME:AddMessage("Unknown Position: 0,0");
   else
      DEFAULT_CHAT_FRAME:AddMessage("X: " .. x .. " Y:" .. y);
   end
end

function akz_ShowVersion(v)
   if (DEFAULT_CHAT_FRAME) then
      DEFAULT_CHAT_FRAME:AddMessage("WowReader version " .. v .. " loaded.");
   else 
      message("WowReader version "  .. v .. " loaded.");
   end
end

function akz_GetCurrentBids()
  local CurTime = GetTime();
  if (CurTime - akz_lastauctionupdatetime < 2) then
    -- do nothing,  this prevents repeated calls to update
  else 
    akz_lastauctionupdatetime = CurTime;
    local MAX = 50;
    local numtoloop = 0;
    
    local numBatchAuctions, totalAuctions = GetNumAuctionItems("list");
    if (totalAuctions > MAX) then 
      numtoloop = MAX;
    else
      numtoloop = totalAuctions;
    end

    local Seen = {};  -- table of items seen this load
    
    for i = 1, numtoloop  do
      local name, texture, count, quality, canUse, level, minBid, minIncrement, buyoutPrice, bidAmount, highBidder =  GetAuctionItemInfo("list",i);
      local itemid,enchant,subid,itemname;
      if (name) then
        itemid,enchant,subid,itemname = akz_GetItemInfoFromLink(GetAuctionItemLink("list",i));
        local item = itemid .. ":" .. subid;
        if (count > 1) then
          price = string.format("%d",minBid / count);
        else 
          price = string.format("%d", minBid);
        end
      	if (not Seen[item]) then  -- wipes existing items seen on previous searches
          akz_prices[item] = {};
          Seen[item] = 1;
      	end
        if (not akz_prices[item]) then
          akz_prices[item] = {};
      	end
      	table.insert(akz_prices[item],price);
      end 
    end
  end 
end
